@@ -8,15 +8,6 @@
 #include "GS_compss.h"
 #include "param_metadata.h"
 
-// Uncomment the following define to get debug information.
-//#define DEBUG_BINDING
-
-#ifdef DEBUG_BINDING
-#define debug_printf(args...) printf(args); fflush(stdout);
-#else
-#define debug_printf(args...) {}
-#endif
-
 using namespace std;
 
 JNIEnv *env;
@@ -26,27 +17,28 @@ JavaVM * jvm;
 
 jobject appId;
 
-jmethodID midAppDir; 		/* ID of the getApplicationDirectory method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
-jmethodID midExecute; 		/* ID of the executeTask method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
-jmethodID midExecuteNew;	/* ID of the executeTask method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
-jmethodID midRegisterCE; 	/* ID of the RegisterCE method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
-jmethodID midEmitEvent; 	/* ID of the EmitEvent method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midAppDir; 		/* ID of the getApplicationDirectory method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midExecute; 		/* ID of the executeTask method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midExecuteNew;	/* ID of the executeTask method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midRegisterCE; 	/* ID of the RegisterCE method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midEmitEvent; 	/* ID of the EmitEvent method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
 
-jmethodID midOpenFile; 		/* ID of the openFile method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
-jmethodID midDeleteFile; 	/* ID of the deleteFile method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midOpenFile; 		/* ID of the openFile method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midCloseFile;
+jmethodID midDeleteFile; 	/* ID of the deleteFile method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
 
-jmethodID midBarrier; 		/* ID of the barrier method in the integratedtoolkit.api.impl.COMPSsRuntimeImpl class  */
+jmethodID midBarrier; 		/* ID of the barrier method in the es.bsc.compss.api.impl.COMPSsRuntimeImpl class  */
 
-jobject jobjParDirIN; 		/* Instance of the integratedtoolkit.types.annotations.parameter.Direction class */
-jobject jobjParDirINOUT; 	/* Instance of the integratedtoolkit.types.annotations.parameter.Direction class */
-jobject jobjParDirOUT; 		/* Instance of the integratedtoolkit.types.annotations.parameter.Direction class */
+jobject jobjParDirIN; 		/* Instance of the es.bsc.compss.types.annotations.parameter.Direction class */
+jobject jobjParDirINOUT; 	/* Instance of the es.bsc.compss.types.annotations.parameter.Direction class */
+jobject jobjParDirOUT; 		/* Instance of the es.bsc.compss.types.annotations.parameter.Direction class */
 
-jobject jobjParStreamSTDIN;     /* Instance of the integratedtoolkit.types.annotations.parameter.Stream class */
-jobject jobjParStreamSTDOUT;    /* Instance of the integratedtoolkit.types.annotations.parameter.Stream class */
-jobject jobjParStreamSTDERR;    /* Instance of the integratedtoolkit.types.annotations.parameter.Stream class */
-jobject jobjParStreamUNSPECIFIED; /* Instance of the integratedtoolkit.types.annotations.parameter.Stream class */
+jobject jobjParStreamSTDIN;     /* Instance of the es.bsc.compss.types.annotations.parameter.Stream class */
+jobject jobjParStreamSTDOUT;    /* Instance of the es.bsc.compss.types.annotations.parameter.Stream class */
+jobject jobjParStreamSTDERR;    /* Instance of the es.bsc.compss.types.annotations.parameter.Stream class */
+jobject jobjParStreamUNSPECIFIED; /* Instance of the es.bsc.compss.types.annotations.parameter.Stream class */
 
-jstring jobjParPrefixEMPTY;     /* Instance of the integratedtoolkit.types.annotations.Constants.PREFIX_EMPTY */
+jstring jobjParPrefixEMPTY;     /* Instance of the es.bsc.compss.types.annotations.Constants.PREFIX_EMPTY */
 
 jclass clsObject; 		/*  java.lang.Object class */
 jmethodID midObjCon; 		/* ID of the java.lang.Object class constructor method */
@@ -86,6 +78,7 @@ JNIEnv* create_vm(JavaVM ** jvm) {
   vector<JavaVMOption> options;
 
   string line; // buffer for line read
+  debug_printf("[   BINDING]  -  @create_vm  -  reading file in JVM_OPTIONS_FILE\n" );
   const char *file = strdup(getenv("JVM_OPTIONS_FILE")); // path to the file with jvm options
   ifstream fin; // input file stream
 
@@ -95,6 +88,8 @@ JNIEnv* create_vm(JavaVM ** jvm) {
       // read in one line at a time
       getline(fin, line);
       // read data from file
+      debug_printf("[   BINDING]  -  @create_vm  -  reading line: \n");
+      debug_printf("[   BINDING]  -  @create_vm  - %s\n", line.data());
       string fileOption = strdup(line.data());
       if (fileOption != "") {
         JavaVMOption *option = new JavaVMOption();
@@ -132,6 +127,7 @@ JNIEnv* create_vm(JavaVM ** jvm) {
           debug_printf("[   BINDING]  -  @create_vm  -  option %s\n", option->optionString);
         } else {
           // It is an environment variable
+          debug_printf("[   BINDING]  -  @create_vm  -  Putting environment variable\n");
           int ret = putenv(strdup(fileOption.data()));
           if (ret < 0) {
             debug_printf("[   BINDING]  -  @create_vm  -  Cannot put environment variable: %s", fileOption.data());
@@ -158,7 +154,7 @@ JNIEnv* create_vm(JavaVM ** jvm) {
   vm_args.options = new JavaVMOption[vm_args.nOptions];
   copy(options.begin(), options.end(), vm_args.options);
   vm_args.ignoreUnrecognized = false;
-
+  debug_printf("[   BINDING]  -  @create_vm  -  Launching JVM\n");
   int ret = JNI_CreateJavaVM(jvm, (void**) &env, &vm_args);
   if (ret < 0){
     debug_printf("[   BINDING]  -  @create_vm  -  Unable to Launch JVM - %i\n", ret);
@@ -177,10 +173,10 @@ void destroy_vm(JavaVM * jvm) {
 }
 
 void init_jni_types() {
-  jclass clsParDir; 		/* integratedtoolkit.types.annotations.parameter.Direction class */
-  jmethodID midParDirCon; 	/* ID of the integratedtoolkit.types.annotations.parameter.Direction class constructor method */
-  jclass clsParStream;          /* integratedtoolkit.types.annotations.parameter.Stream class */
-  jmethodID midParStreamCon;    /* integratedtoolkit.types.annotations.parameter.Stream class constructor method */
+  jclass clsParDir; 		/* es.bsc.compss.types.annotations.parameter.Direction class */
+  jmethodID midParDirCon; 	/* ID of the es.bsc.compss.types.annotations.parameter.Direction class constructor method */
+  jclass clsParStream;          /* es.bsc.compss.types.annotations.parameter.Stream class */
+  jmethodID midParStreamCon;    /* es.bsc.compss.types.annotations.parameter.Stream class constructor method */
 
   debug_printf ("[   BINDING]  -  @Init JNI Methods\n");
 
@@ -220,7 +216,14 @@ void init_jni_types() {
   }
 
   // openFile method
-  midOpenFile = env->GetMethodID(clsITimpl, "openFile", "(Ljava/lang/String;Lintegratedtoolkit/types/annotations/parameter/Direction;)Ljava/lang/String;");
+  midOpenFile = env->GetMethodID(clsITimpl, "openFile", "(Ljava/lang/String;Les/bsc/compss/types/annotations/parameter/Direction;)Ljava/lang/String;");
+  if (env->ExceptionOccurred()) {
+    env->ExceptionDescribe();
+    exit(1);
+  }
+
+  // closeFile method
+  midCloseFile = env->GetMethodID(clsITimpl, "closeFile", "(Ljava/lang/String;Les/bsc/compss/types/annotations/parameter/Direction;)V");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
@@ -244,12 +247,12 @@ void init_jni_types() {
 
   debug_printf ("[   BINDING]  -  @Init JNI Direction Types\n");
 
-  clsParDir = env->FindClass("integratedtoolkit/types/annotations/parameter/Direction");
+  clsParDir = env->FindClass("es/bsc/compss/types/annotations/parameter/Direction");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
   }
-  midParDirCon = env->GetStaticMethodID(clsParDir, "valueOf", "(Ljava/lang/String;)Lintegratedtoolkit/types/annotations/parameter/Direction;");
+  midParDirCon = env->GetStaticMethodID(clsParDir, "valueOf", "(Ljava/lang/String;)Les/bsc/compss/types/annotations/parameter/Direction;");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
@@ -274,12 +277,12 @@ void init_jni_types() {
 
   debug_printf ("[   BINDING]  -  @Init JNI Stream Types\n");
 
-  clsParStream = env->FindClass("integratedtoolkit/types/annotations/parameter/Stream");
+  clsParStream = env->FindClass("es/bsc/compss/types/annotations/parameter/Stream");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
   }
-  midParStreamCon = env->GetStaticMethodID(clsParStream, "valueOf", "(Ljava/lang/String;)Lintegratedtoolkit/types/annotations/parameter/Stream;");
+  midParStreamCon = env->GetStaticMethodID(clsParStream, "valueOf", "(Ljava/lang/String;)Les/bsc/compss/types/annotations/parameter/Stream;");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
@@ -408,26 +411,28 @@ void init_jni_types() {
 
 
 void process_param(void **params, int i, jobjectArray jobjOBJArr) {
-  // params is of the form:     value type direction
+  // params     is of the form: value type direction stream prefix
   // jobjOBJArr is of the form: value type direction stream prefix
 
   debug_printf("[   BINDING]  -  @process_param\n");
 
-  void *parVal = params[3*i];
-  int parType = *(int*)params[3*i + 1];
-  int parDirect = *(int*)params[3*i + 2];
+  void *parVal = params[5*i];
+  int parType = *(int*)params[5*i + 1];
+  int parDirect = *(int*)params[5*i + 2];
+  int parStream = *(int*)params[5*i + 3];
+  void *parPrefix = params[5*i + 4];
 
   int pv = 5*i, pt = 5*i + 1, pd = 5*i + 2, ps = 5*i + 3, pp = 5*i + 4;
 
-  jclass clsParType = NULL; /* integratedtoolkit.types.annotations.parameter.DataType class */
-  clsParType = env->FindClass("integratedtoolkit/types/annotations/parameter/DataType");
+  jclass clsParType = NULL; /* es.bsc.compss.types.annotations.parameter.DataType class */
+  clsParType = env->FindClass("es/bsc/compss/types/annotations/parameter/DataType");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
   }
 
-  jmethodID midParTypeCon = NULL; /* ID of the integratedtoolkit.api.COMPSsRuntime$DataType class constructor method */
-  midParTypeCon = env->GetStaticMethodID(clsParType, "valueOf", "(Ljava/lang/String;)Lintegratedtoolkit/types/annotations/parameter/DataType;");
+  jmethodID midParTypeCon = NULL; /* ID of the es.bsc.compss.api.COMPSsRuntime$DataType class constructor method */
+  midParTypeCon = env->GetStaticMethodID(clsParType, "valueOf", "(Ljava/lang/String;)Les/bsc/compss/types/annotations/parameter/DataType;");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
@@ -618,10 +623,27 @@ void process_param(void **params, int i, jobjectArray jobjOBJArr) {
   }
 
   // Add param stream
-  env->SetObjectArrayElement(jobjOBJArr, ps, jobjParStreamUNSPECIFIED);
+  debug_printf ("[   BINDING]  -  @process_param  -  ENUM STREAM: %d\n", (enum stream) parStream);
+  switch ((enum stream) parStream) {
+    case STD_IN:
+      env->SetObjectArrayElement(jobjOBJArr, ps, jobjParStreamSTDIN);
+      break;
+    case STD_OUT:
+      env->SetObjectArrayElement(jobjOBJArr, ps, jobjParStreamSTDOUT);
+      break;
+    case STD_ERR:
+      env->SetObjectArrayElement(jobjOBJArr, ps, jobjParStreamSTDERR);
+      break;
+    default:
+      env->SetObjectArrayElement(jobjOBJArr, ps, jobjParStreamUNSPECIFIED);
+      break;
+  }
 
   // Add param prefix
-  env->SetObjectArrayElement(jobjOBJArr, pp, jobjParPrefixEMPTY);
+  debug_printf ("[   BINDING]  -  @process_param  -  PREFIX: %s\n", *(char **)parPrefix);
+  jstring jobjParPrefix = env->NewStringUTF(*(char **)parPrefix);
+  env->SetObjectArrayElement(jobjOBJArr, pp, jobjParPrefix);
+  //env->SetObjectArrayElement(jobjOBJArr, pp, jobjParPrefixEMPTY);
 }
 
 // ******************************
@@ -643,7 +665,7 @@ void GS_On()
   }
 
   //Obtaining Classes
-  clsITimpl = env->FindClass("integratedtoolkit/api/impl/COMPSsRuntimeImpl");
+  clsITimpl = env->FindClass("es/bsc/compss/api/impl/COMPSsRuntimeImpl");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     printf("[   BINDING]  -  @GS_On  -  Error looking for the COMPSsRuntimeImpl class\n");
@@ -709,6 +731,21 @@ void GS_On()
   }
 }
 
+int check_and_attach()
+{
+  jint res = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
+  if (res == JNI_EDETACHED) {
+        debug_printf("[   BINDING]  -  @check_an_attach - Attaching\n");
+        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
+            printf("Failed to attach to the JVM");
+        }else{
+	    return 1;
+        }
+  } else {
+        // attached
+	return 0;
+  }
+}
 
 void GS_Off()
 {
@@ -717,6 +754,8 @@ void GS_Off()
   jmethodID midStopIT = NULL;
   jmethodID midNoMoreTasksIT = NULL;
 
+  int isAttached = check_and_attach();
+
   midNoMoreTasksIT = env->GetMethodID(clsITimpl, "noMoreTasks", "(Ljava/lang/Long;Z)V");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
@@ -728,19 +767,19 @@ void GS_Off()
     env->ExceptionDescribe();
     exit(1);
   }
-
+  debug_printf("[   BINDING]  -  @Off - Waiting to end tasks\n");
   env->CallVoidMethod(jobjIT, midNoMoreTasksIT, appId, "TRUE");
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
   }
-
+  debug_printf("[   BINDING]  -  @Off - Stopping runtime\n");
   env->CallVoidMethod(jobjIT, midStopIT, "TRUE"); //Calling the method and passing IT Object as parameter
   if (env->ExceptionOccurred()) {
     env->ExceptionDescribe();
     exit(1);
   }
-
+  debug_printf("[   BINDING]  -  @Off - Removing JVM\n");
   destroy_vm(jvm);  // Release jvm resources -- Does not work properly --> JNI bug: not releasing properly the resources, so it is not possible to recreate de JVM.
   // delete jvm;    // free(): invalid pointer: 0x00007fbc11ba8020 ***
   jvm = NULL;
@@ -753,6 +792,7 @@ void GS_Get_AppDir(char **buf)
   const char *cstr;
   jstring jstr = NULL;
   jboolean isCopy;
+  int isAttached = check_and_attach();
 
   jstr = (jstring)env->CallObjectMethod(jobjIT, midAppDir);
   if (env->ExceptionOccurred()) {
@@ -765,12 +805,17 @@ void GS_Get_AppDir(char **buf)
   cstr = env->GetStringUTFChars(jstr, &isCopy);
   *buf = strdup(cstr);
   env->ReleaseStringUTFChars(jstr, cstr);
+  if (isAttached == 1){
+  	jvm->DetachCurrentThread();
+  }
 
   debug_printf("[   BINDING]  -  @GS_Get_AppDir  -  directory name: %s\n", *buf);
 }
 
 void GS_ExecuteTask(long _appId, char *class_name, char *method_name, int priority, int has_target, int num_params, void **params)
 {
+  int isAttached = check_and_attach();
+
   jobjectArray jobjOBJArr; /*  array of Objects to be passed to executeTask */
 
   debug_printf ("[   BINDING]  -  @GS_ExecuteTask\n");
@@ -793,12 +838,18 @@ void GS_ExecuteTask(long _appId, char *class_name, char *method_name, int priori
     env->ExceptionDescribe();
     exit(1);
   }
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
 }
 
 void GS_ExecuteTaskNew(long _appId, char *signature, int priority, int num_nodes, int replicated, int distributed, int has_target, int num_params, void **params)
 {
+
   jobjectArray jobjOBJArr; /* array of Objects to be passed to executeTask */
 
+  int isAttached = check_and_attach();
+
   debug_printf ("[   BINDING]  -  @GS_ExecuteTaskNew\n");
 
   bool _priority = false;
@@ -816,7 +867,7 @@ void GS_ExecuteTaskNew(long _appId, char *signature, int priority, int num_nodes
   jobjOBJArr = (jobjectArray)env->NewObjectArray(num_params*5, clsObject, env->NewObject(clsObject,midObjCon));
 
   for (int i = 0; i < num_params; i++) {
-    debug_printf("[   BINDING]  -  @GS_ExecuteTask  -  Processing pos %d\n", i);
+    debug_printf("[   BINDING]  -  @GS_ExecuteTaskNew  -  Processing pos %d\n", i);
     process_param(params, i, jobjOBJArr);
   }
 
@@ -833,10 +884,15 @@ void GS_ExecuteTaskNew(long _appId, char *signature, int priority, int num_nodes
     env->ExceptionDescribe();
     exit(1);
   }
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
 }
 
 void GS_RegisterCE(char *CESignature, char *ImplSignature, char *ImplConstraints, char *ImplType, int num_params, char **ImplTypeArgs)
 {
+  int isAttached = check_and_attach();
+
   debug_printf ("[   BINDING]  -  @GS_RegisterCE\n");
   //debug_printf ("[   BINDING]  -  @GS_RegisterCE - CESignature:     %s\n", CESignature);
   //debug_printf ("[   BINDING]  -  @GS_RegisterCE - ImplSignature:   %s\n", ImplSignature);
@@ -851,7 +907,6 @@ void GS_RegisterCE(char *CESignature, char *ImplSignature, char *ImplConstraints
     jstring tmp = env->NewStringUTF(ImplTypeArgs[i]);
     env->SetObjectArrayElement(implArgs, i, tmp);
   }
-
   env->CallVoidMethod(jobjIT, midRegisterCE, env->NewStringUTF(CESignature),
                                              env->NewStringUTF(ImplSignature),
                                              env->NewStringUTF(ImplConstraints),
@@ -861,16 +916,23 @@ void GS_RegisterCE(char *CESignature, char *ImplSignature, char *ImplConstraints
       env->ExceptionDescribe();
       exit(1);
   }
+  if (isAttached==1){
+ 	 jvm->DetachCurrentThread();
+  }
+
   debug_printf("[   BINDING]  -  @GS_RegisterCE  -  Task registered: %s\n", CESignature);
 }
 
 
 void GS_Get_File(char *file_name, int mode, char **buf)
 {
+
   const char *cstr;
   jstring jstr = NULL;
   jboolean isCopy;
 
+  int isAttached = check_and_attach();
+
   switch ((enum direction) mode) {
     case in_dir:
       jstr = (jstring)env->CallObjectMethod(jobjIT, midOpenFile, env->NewStringUTF(file_name), jobjParDirIN);
@@ -893,35 +955,79 @@ void GS_Get_File(char *file_name, int mode, char **buf)
   cstr = env->GetStringUTFChars(jstr, &isCopy);
   *buf = strdup(cstr);
   env->ReleaseStringUTFChars(jstr, cstr);
-
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
   debug_printf("[   BINDING]  -  @GS_Get_File  -  COMPSs filename: %s\n", *buf);
 }
 
+void GS_Close_File(char *file_name, int mode) {
+
+  int isAttached = check_and_attach();
+
+  switch ((enum direction) mode) {
+    case in_dir:
+      env->CallVoidMethod(jobjIT, midCloseFile, env->NewStringUTF(file_name), jobjParDirIN);
+      break;
+    case out_dir:
+      env->CallVoidMethod(jobjIT, midCloseFile, env->NewStringUTF(file_name), jobjParDirOUT);
+      break;
+    case inout_dir:
+      env->CallVoidMethod(jobjIT, midCloseFile, env->NewStringUTF(file_name), jobjParDirINOUT);
+      break;
+    default:
+      break;
+  }
+
+  if (env->ExceptionOccurred()) {
+      env->ExceptionDescribe();
+      exit(1);
+  }
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
+  debug_printf("[   BINDING]  -  @GS_Close_File  -  COMPSs filename: %s\n", file_name);
+}
+
 
 void GS_Delete_File(char *file_name, int **buf)
 {
+  int isAttached = check_and_attach();
+
   env->CallVoidMethod(jobjIT, midDeleteFile, env->NewStringUTF(file_name));
   if (env->ExceptionOccurred()) {
       env->ExceptionDescribe();
       exit(1);
   }
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
+
   debug_printf("[   BINDING]  -  @GS_Delete_File  -  COMPSs filename: %s\n", file_name);
 }
 
 
 void GS_Barrier(long _appId)
 {
+  int isAttached = check_and_attach();
+
   env->CallVoidMethod(jobjIT, midBarrier, appId);
   if (env->ExceptionOccurred()) {
       env->ExceptionDescribe();
       exit(1);
   }
+
+  if (isAttached==1){
+    jvm->DetachCurrentThread();
+  }
   debug_printf("[   BINDING]  -  @GS_Barrier  -  APP id: %lu", appId);
 }
 
 
 void GS_EmitEvent(int type, long id)
 {
+  int isAttached = check_and_attach();
+
   if ( (type < 0 ) or (id < 0) ) {
     debug_printf ("[   BINDING]  -  @GS_EmitEvent  -  Error: event type and ID must be positive integers, but found: type: %u, ID: %lu\n", type, id);
     exit(1);
@@ -933,4 +1039,7 @@ void GS_EmitEvent(int type, long id)
       exit(1);
     }
   }
+  if (isAttached==1){
+  	jvm->DetachCurrentThread();
+  }
 }
