@@ -1,9 +1,10 @@
- 
-#include <stdio.h>
 #include "executor.h"
 
+std::map<std::string, void*> cache;
+std::map<std::string, int> types;
+
 int main(int argc, char **argv) {
-    int out = execute(argc, argv);
+    int out = execute(argc, argv, cache, types, 1);
     if (out == 0){
         printf("Task executed successfully");
     }else{
