@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2.02-2.27 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -31,31 +31,20 @@ compssmodule = Extension('compss',
         extra_compile_args = ['-fPIC'],
         sources = ['src/ext/compssmodule.c'])
 
-shm_manager = Extension('shm_manager',
-	include_dirs = ['src/ext'],
-	sources = ['src/ext/shm_manager.cc'],
-    language='c++'
-)
-
-persistent_cache = Extension('persistent_cache',
-	include_dirs = ['src/ext'],
-	sources = ['src/ext/persistent_cache.cc']
-)
-
 thread_affinity = Extension('thread_affinity',
 	include_dirs = ['src/ext'],
 	sources = ['src/ext/thread_affinity.cc']
 )
 
 setup (name='pycompss',
-	version='2.1',
+	version='2.2',
 	description='Python Binding for COMP Superscalar Runtime',
 	long_description=open('README.txt').read(),
 	author='COMPSs Team',
 	author_email='support-compss@bsc.es',
 	url='http://compss.bsc.es',
-	license='Apache 2.1',
+	license='Apache 2.2',
     package_dir={'pycompss':'src/pycompss'},
 	packages=['', 'pycompss', 'pycompss.api', 'pycompss.runtime', 'pycompss.worker', 'pycompss.util', 'pycompss.util.serialization', 'pycompss.api.dummy', 'pycompss.functions', 'pycompss.matlib', 'pycompss.matlib.algebra', 'pycompss.matlib.classification', 'pycompss.matlib.clustering'],
 	package_data={'' : ['log/logging.json', 'log/logging.json.debug', 'log/logging.json.off', 'bin/worker_python.sh']},
-	ext_modules=[compssmodule, persistent_cache, thread_affinity])
+	ext_modules=[compssmodule, thread_affinity])
