@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: fconejer
 
+"""
 PyCOMPSs API - Implement (Versioning)
 =====================================
     This file contains the class constraint, needed for the implement
@@ -95,9 +94,10 @@ class implement(object):
             ce_signature = anotherClass + '.' + anotherMethod
             coreElement.set_ce_signature(ce_signature)
 
-            # This is not needed since the arguments are already set by the task decorator.
-            #implArgs = [anotherClass, anotherMethod]
-            #coreElement.set_implTypeArgs(implArgs)
+            # This is not needed since the arguments are already set by the
+            # task decorator.
+            # implArgs = [anotherClass, anotherMethod]
+            # coreElement.set_implTypeArgs(implArgs)
 
             coreElement.set_implType("METHOD")
             func.__to_register__ = coreElement
@@ -109,7 +109,6 @@ class implement(object):
             # worker code
             pass
 
-
         @wraps(func)
         def implement_f(*args, **kwargs):
             # This is executed only when called.
