@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: fconejer
 
+"""
 PyCOMPSs API - OMPSS
 ==================
     This file contains the class constraint, needed for the OmpSs task
@@ -43,7 +42,8 @@ class ompss(object):
         self.kwargs = kwargs
         logger.debug("Init @ompss decorator...")
 
-        # Get the computing nodes -- This parameter will have to go down until execution when invoked.
+        # Get the computing nodes: This parameter will have to go down until
+        # execution when invoked.
         if 'computingNodes' not in self.kwargs:
             self.kwargs['computingNodes'] = 1
         else:
@@ -113,13 +113,13 @@ class ompss(object):
             # worker code
             pass
 
-
         @wraps(func)
         def ompss_f(*args, **kwargs):
             # This is executed only when called.
             logger.debug("Executing ompss_f wrapper.")
 
-            # Set the computingNodes variable in kwargs for its usage in @task decorator
+            # Set the computingNodes variable in kwargs for its usage
+            # in @task decorator
             kwargs['computingNodes'] = self.kwargs['computingNodes']
 
             if len(args) > 0:
