@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: fconejer
 
+"""
 PyCOMPSs Functions: Elapsed time decorator
 ==========================================
     This file defines the time it decorator to be used over the task decorator.
@@ -24,11 +23,13 @@ PyCOMPSs Functions: Elapsed time decorator
 from decorator import decorator
 import time
 
+
 @decorator  # Mandatory in order to preserver the argspec
 def timeit(func, *a, **k):
     """
     Elapsed time decorator.
-    :param func: Function to be measured (can be a decorated function, usually with @task decorator).
+    :param func: Function to be measured (can be a decorated function,
+     usually with @task decorator).
     :param a: args
     :param k: kwargs
     :return: a list with [the function result, The elapsed time]
@@ -36,4 +37,4 @@ def timeit(func, *a, **k):
     ts = time.time()
     result = func(*a, **k)
     te = time.time()
-    return [result, (te-ts)]
\ No newline at end of file
+    return [result, (te-ts)]
