@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: scorella
 
+"""
 PyCOMPSs Functions: Data generators
 ===================================
     This file defines the common data producing functions.
@@ -24,12 +23,14 @@ PyCOMPSs Functions: Data generators
 
 def sort(iterable, comp=None, key=None, reverse=False):
     """
-    Apply function of two arguments cumulatively to the items of data, from left to right,
-    so as to reduce the iterable to a single value.
+    Apply function of two arguments cumulatively to the items of data, from
+    left to right, so as to reduce the iterable to a single value.
     :param iterable: data.
     :param comp: specifies a custom comparison function of two arguments.
-    :param key: specifies a function of one argument that is used to extract a comparison key from each list element.
-    :param reverse: if set to True, then the list elements are sorted as if each comparison were reversed.
+    :param key: specifies a function of one argument that is used to extract
+                a comparison key from each list element.
+    :param reverse: if set to True, then the list elements are sorted as if
+                    each comparison were reversed.
     :return: a new sorted list from the items in iterable.
     """
     try:
