@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: scorella
 
+"""
 PyCOMPSs Mathematical Library: Clustering: KMeans
 =================================================
     This file contains the KMeans algorithm.
@@ -86,7 +85,7 @@ def has_converged(mu, oldmu, epsilon, iter, maxIterations):
             else:
                 return False
         else:
-            # detencion pq se ha alcanzado el maximo de iteraciones
+            # Reached max iterations
             return True
 
 
@@ -155,7 +154,8 @@ def init(X, k, mode):
         return init_random(dim, k)
 
 
-def kmeans(data, k, numFrag=-1, maxIterations=10, epsilon=1e-4, initMode='random'):
+def kmeans(data, k, numFrag=-1, maxIterations=10, epsilon=1e-4,
+           initMode='random'):
     """
     kmeans: starting with a set of randomly chosen initial centers,
     one repeatedly assigns each imput point to its nearest center, and
