@@ -1,5 +1,5 @@
 #
-#  Copyright 2002-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: fconejer
 
+"""
 PyCOMPSs Binding - Interactive Mode Helpers
 ===========================================
 Provides auxiliar methods for the interactive mode.
@@ -23,10 +22,10 @@ Provides auxiliar methods for the interactive mode.
 
 import os
 
-separators = {'globals_separator':"### GLOBALS ###",      # for user defined lines in the entire/global scope
-              'classes_separator':'### CLASSES ###',      # for user defined classes
-              'functions_separator':"### FUNCTIONS ###",  # for user defined functions (that are not decorated)
-              'tasks_separator':"### TASKS ###"}          # for user defined tasks
+separators = {'globals_separator': "### GLOBALS ###",      # for user defined lines in the entire/global scope
+              'classes_separator': '### CLASSES ###',      # for user defined classes
+              'functions_separator': "### FUNCTIONS ###",  # for user defined functions (that are not decorated)
+              'tasks_separator': "### TASKS ###"}          # for user defined tasks
 
 ####################################################################
 ################## MAIN FUNCTION ###################################
@@ -51,12 +50,12 @@ def updateTasksCodeFile(f, filePath):
     print("Task definition detected.")
 
     # Intercept the code
-    imports = getIPythonImports()     ######## [import\n, import\n, ...]
-    globalCode = getIPythonGlobals()  ######## [var\n, var\n, ...]
-    classesCode = getClasses()        ######## {'name': str(line\nline\n...)}
-    functionsCode = getFunctions()    ######## {'name': str(line\nline\n...)}
-    taskCode = getTaskCode(f)         ######## {'name': str(line\nline\n...)}
-    oldCode = getOldCode(filePath)    ######## {'imports':[import\n, import\n, ...], 'tasks':{'name':str(line\nline\n...), 'name':str(line\nline\n...), ...}}
+    imports = getIPythonImports()     # [import\n, import\n, ...]
+    globalCode = getIPythonGlobals()  # [var\n, var\n, ...]
+    classesCode = getClasses()        # {'name': str(line\nline\n...)}
+    functionsCode = getFunctions()    # {'name': str(line\nline\n...)}
+    taskCode = getTaskCode(f)         # {'name': str(line\nline\n...)}
+    oldCode = getOldCode(filePath)    # {'imports':[import\n, import\n, ...], 'tasks':{'name':str(line\nline\n...), 'name':str(line\nline\n...), ...}}
 
     # Look for new/modified pieces of code. Compares the existing code with the new additions.
     newImports = updateImports(imports, oldCode['imports'])
@@ -69,13 +68,11 @@ def updateTasksCodeFile(f, filePath):
     updateCodeFile(newImports, newGlobals, newClasses, newFunctions, newTasks, filePath)
 
 
-
-
 ####################################################################
 ################ AUXILIAR METHODS ##################################
 ####################################################################
 
-###  * CODE INTERCEPTION FUNCTIONS
+# CODE INTERCEPTION FUNCTIONS
 
 def createTasksCodeFile(filePath):
     """
@@ -98,10 +95,12 @@ def createTasksCodeFile(filePath):
 def getRawCode():
     """
     Retrieve the raw code from jupyter
-    :return: the list of the blocks defined by the user that are currently loaded in globals
+    :return: the list of the blocks defined by the user that are currently
+             loaded in globals
     """
-    # print globals()['In'] # is not in this scope
-    ipython = globals()['__builtins__']['get_ipython']()  # retrieve the self of ipython where to look
+    # globals()['In'] # is not in this scope
+    # Retrieve the self of ipython where to look
+    ipython = globals()['__builtins__']['get_ipython']()
     # If you want to show the contents of the ipython object for analysis
     # file.write(str(ipython.__dict__))
     # import pprint
@@ -119,7 +118,8 @@ def getIPythonImports():
     imports = []
     for i in raw_code:
         # Each i can have more than one line (jupyter-notebook block)
-        # We only get the lines that start with from or import and do not have blank spaces before.
+        # We only get the lines that start with from or import and do not
+        # have blank spaces before.
         lines = i.split('\n')
         for l in lines:
             if l.startswith("from") or l.startswith("import"):
@@ -136,7 +136,8 @@ def getIPythonGlobals():
     globLines = {}
     for i in raw_code:
         # Each i can have more than one line (jupyter-notebook block)
-        # We only get the lines that start with from or import and do not have blank spaces before.
+        # We only get the lines that start with from or import and do not
+        # have blank spaces before.
         lines = i.split('\n')
         for l in lines:
             # if the line starts without spaces and is a variable assignation
@@ -148,7 +149,7 @@ def getIPythonGlobals():
 
 def isVariableAssignation(line):
     '''
-    This function is used to check if a line of code represents a variable assignation.
+    This function is used to check if a line of code represents a variable assignation:
     * if contains a '=' (assignation) and does not start with import, nor @, nor def, nor class
     * then it is ==> is a global variable assignation
     :param line: Line to parse
@@ -156,19 +157,27 @@ def isVariableAssignation(line):
     '''
     if '=' in line:
         parts = line.split()
-        if not (line.startswith("from") or line.startswith("import") or line.startswith("@") or line.startswith("def")
-                or line.startswith("class") or '(' in line or ')' in line) and len(parts) == 3 and parts[1] == '=':
-            return True  # It is actually an assignation
+        if not (line.startswith("from") or
+                line.startswith("import") or
+                line.startswith("@") or
+                line.startswith("def") or
+                line.startswith("class") or '(' in line or ')' in line) \
+                and len(parts) == 3 and parts[1] == '=':
+            # It is actually an assignation
+            return True
         else:
-            return False # It is an import/function/decorator/class definition
+            # It is an import/function/decorator/class definition
+            return False
     else:
-        return False     # Not an assignation if does not contain '='
+        # Not an assignation if does not contain '='
+        return False
 
 
 def getClasses():
     """
     Finds the user defined classes in the code
-    :return: A dictionary with the user classes code: {'name': str(line\nline\n...)}
+    :return: A dictionary with the user classes code:
+             {'name': str(line\nline\n...)}
     """
     raw_code = getRawCode()
     classes = {}
@@ -179,9 +188,12 @@ def getClasses():
         for l in lines:
             if l.startswith('class'):
                 # Class header: find name and include it in the functions dict
-                header = filter(None, l.split(" "))  # split and remove empty spaces
-                className = header[1].split("(")[0].strip()  # the name may be followed by the parameters parenthesis
-                classes[className] = [l + '\n']  # create an entry in the functions dict
+                # split and remove empty spaces
+                header = filter(None, l.split(" "))
+                # the name may be followed by the parameters parenthesis
+                className = header[1].split("(")[0].strip()
+                # create an entry in the functions dict
+                classes[className] = [l + '\n']
                 classFound = True
             elif (l.startswith("  ") or (l.startswith("\t")) or (l.startswith('\n')) or (l == '')) and classFound:
                 # class body: append
@@ -189,15 +201,17 @@ def getClasses():
             else:
                 classFound = False
     # Plain classes content (from {key: [line, line,...]} to {key: line\nline}).
-    for k,v in classes.iteritems():
-        classes[k] = ''.join(v).strip()      # Collapse all lines into a single one
+    for k, v in classes.iteritems():
+        # Collapse all lines into a single one
+        classes[k] = ''.join(v).strip()
     return classes
 
 
 def getFunctions():
     """
     Finds the user defined functions in the code
-    :return: A dictionary with the user functions code: {'name': str(line\nline\n...)}
+    :return: A dictionary with the user functions code:
+             {'name': str(line\nline\n...)}
     """
     raw_code = getRawCode()
     functions = {}
@@ -212,16 +226,21 @@ def getFunctions():
                 # The followihg function detected will be a task --> ignore
                 isTask = True
             if l.startswith("def") and not isTask:
-                # A function which is not a task has been defined --> capture with isFunction boolean
-                # Restore the isTask boolean to control if another task is defined in the same block.
+                # A function which is not a task has been defined --> capture
+                # with isFunction boolean
+                # Restore the isTask boolean to control if another task is
+                # defined in the same block.
                 isFunction = True
                 isTask = False
             if isFunction:
                 if l.startswith("def"):
                     # Function header: find name and include it in the functions dict
-                    header = filter(None, l.split(" "))  # split and remove empty spaces
-                    funcName = header[1].split("(")[0].strip()  # the name may be followed by the parameters parenthesis
-                    functions[funcName] = [l + '\n']  # create an entry in the functions dict
+                    # split and remove empty spaces
+                    header = filter(None, l.split(" "))
+                    # the name may be followed by the parameters parenthesis
+                    funcName = header[1].split("(")[0].strip()
+                    # create an entry in the functions dict
+                    functions[funcName] = [l + '\n']
                     functionFound = True
                 elif (l.startswith("  ") or (l.startswith("\t")) or (l.startswith('\n')) or (l == '')) and functionFound:
                     # Function body: append
@@ -229,7 +248,7 @@ def getFunctions():
                 else:
                     functionFound = False
     # Plain functions content (from {key: [line, line,...]} to {key: line\nline}).
-    for k,v in functions.iteritems():
+    for k, v in functions.iteritems():
         functions[k] = ''.join(v).strip()      # Collapse all lines into a single one
     return functions
 
@@ -247,7 +266,7 @@ def getTaskCode(f):
     for line in lines:
         if line.startswith('def'):
             name = line.replace('(', ' (').split(' ')[1].strip()
-    return {name:taskCode}
+    return {name: taskCode}
 
 
 def clean(linesList):
@@ -319,7 +338,8 @@ def getOldCode(filePath):
     # Process globals
     globs = {}
     if len(fileGlobals) != 0:
-        collapsed = ''.join(fileGlobals).strip()  # Collapse all lines into a single one
+        # Collapse all lines into a single one
+        collapsed = ''.join(fileGlobals).strip()
         scattered = collapsed.split('\n')
         # Add classes to dictionary by class name:
         for g in scattered:
@@ -330,8 +350,10 @@ def getOldCode(filePath):
 
     # Process classes
     classes = {}
-    collapsed = ''.join(fileClasses).strip()      # Collapse all lines into a single one
-    # Then split by "class" and filter the empty results, then iterate concatenating "class" to all results.
+    # Collapse all lines into a single one
+    collapsed = ''.join(fileClasses).strip()
+    # Then split by "class" and filter the empty results, then iterate
+    # concatenating "class" to all results.
     cls = [('class' + l) for l in filter(None, collapsed.split('class'))]
     # Add classes to dictionary by class name:
     for c in cls:
@@ -341,9 +363,11 @@ def getOldCode(filePath):
 
     # Process functions
     functions = {}
-    collapsed = ''.join(fileFunctions).strip()      # Collapse all lines into a single one
-    # Then split by "def" and filter the empty results, then iterate concatenating "def" to all results.
-    funcs = [('def' + l) for l in filter(None, collapsed.split('def'))]
+    # Collapse all lines into a single one
+    collapsed = ''.join(fileFunctions).strip()
+    # Then split by "def" and filter the empty results, then iterate
+    # concatenating "def" to all results.
+    funcs = [('def ' + l) for l in filter(None, collapsed.split('def '))]
     # Add functions to dictionary by function name:
     for f in funcs:
         funcCode = f.strip()
@@ -352,26 +376,40 @@ def getOldCode(filePath):
 
     # Process tasks
     tasks = {}
-    collapsed = ''.join(fileTasks).strip()  # Collapse all lines into a single one
-    # Then split by "@" and filter the empty results, then iterate concatenating "@" to all results.
-    # TODO CHECK IF MULTIPLE DECORATORS CAN BE SUPPORTED
+    # Collapse all lines into a single one
+    collapsed = ''.join(fileTasks).strip()
+    # Then split by "@" and filter the empty results, then iterate
+    # concatenating "@" to all results.
     tsks = [('@' + l) for l in filter(None, collapsed.split('@'))]
-    # Add functions to dictionary by function name:
+    # Take into account that other decorators my be over @task, so it is
+    # necessary to collapse the function stack
+    prefixes = ("@implement", "@constraint", "@decaf", "@mpi", "@ompss", "@binary", "@opencl")
+    tsksStacked = []
+    tsk = ""
     for t in tsks:
-        taskCode = t.strip()   # Example: '@task(returns=int)\ndef mytask(v):\n    return v+1'
+        if any(map(t.startswith, prefixes)):
+            tsk += t
+        if t.startswith("@task"):
+            tsk += t
+            tsksStacked.append(tsk)
+            tsk = ""
+    # Add functions to dictionary by function name:
+    for t in tsksStacked:
+        # Example: '@task(returns=int)\ndef mytask(v):\n    return v+1'
+        taskCode = t.strip()
         taskHeader = t.split('\ndef')[1]
         taskName = taskHeader.replace('(', ' (').split(' ')[1].strip()
         tasks[taskName] = taskCode
 
-    return {'imports': fileImports, 'globals': fileGlobals, 'classes':classes, 'functions':functions, 'tasks': tasks}
+    return {'imports': fileImports, 'globals': fileGlobals, 'classes': classes, 'functions': functions, 'tasks': tasks}
 
 
-###  * CODE UPDATE FUNCTIONS
+# CODE UPDATE FUNCTIONS
 
 def updateImports(newImports, oldImports):
     """
-    Compare the old imports against the new ones and returns the old imports with the new imports that did
-    not existed previously
+    Compare the old imports against the new ones and returns the old imports
+    with the new imports that did not existed previously
     :param newImports: All new imports
     :param oldImports: All old imports
     :return: A list of imports as strings.
@@ -391,8 +429,8 @@ def updateImports(newImports, oldImports):
 
 def updateGlobals(newGlobals, oldGlobals):
     """
-    Compare the old globals against the new ones and returns the old globals with the new globals that did
-    not existed previously
+    Compare the old globals against the new ones and returns the old globals
+    with the new globals that did not existed previously
     :param newGlobals: All new globals
     :param oldGlobals: All old globals
     :return: A list of globals as strings.
@@ -401,7 +439,7 @@ def updateGlobals(newGlobals, oldGlobals):
         return newGlobals
     else:
         for gName in newGlobals.keys():
-            if oldGlobals.has_key(gName) and (not newGlobals[gName] == oldGlobals[gName]):
+            if gName in oldGlobals and (not newGlobals[gName] == oldGlobals[gName]):
                 print "WARNING! Global variable " + gName + " has been redefined with changes (the previous will be deprecated)."
             oldGlobals[gName] = newGlobals[gName]
         return oldGlobals
@@ -409,17 +447,19 @@ def updateGlobals(newGlobals, oldGlobals):
 
 def updateClasses(newClasses, oldClasses):
     """
-    Compare the old classes against the new ones. This function is essential due to the fact that a jupyter-notebook
-    user may rewrite a function and the latest version is the one that needs to be kept.
-    :param newClasses: dictionary containing all classes (on its last version)
+    Compare the old classes against the new ones. This function is essential
+    due to the fact that a jupyter-notebook user may rewrite a function and
+    the latest version is the one that needs to be kept.
+    :param newClasses: dictionary containing all classes (last version)
     :param oldClasses: dictionary containing the existing classes.
-    return: dictionary with the merging result (keeping all classes and updating the old ones).
+    return: dictionary with the merging result (keeping all classes and
+            updating the old ones).
     """
     if len(oldClasses) == 0:
         return newClasses
     else:
         for cName in newClasses.keys():
-            if oldClasses.has_key(cName) and (not newClasses[cName] == oldClasses[cName]):
+            if cName in oldClasses and (not newClasses[cName] == oldClasses[cName]):
                 print "WARNING! Class " + cName + " has been redefined with changes (the previous will be deprecated)."
             oldClasses[cName] = newClasses[cName]
         return oldClasses
@@ -427,17 +467,19 @@ def updateClasses(newClasses, oldClasses):
 
 def updateFunctions(newFunctions, oldFunctions):
     """
-    Compare the old functions against the new ones. This function is essential due to the fact that a jupyter-notebook
-    user may rewrite a function and the latest version is the one that needs to be kept.
-    :param newFunctions: dictionary containing all functions (on its last version)
+    Compare the old functions against the new ones. This function is essential
+    due to the fact that a jupyter-notebook user may rewrite a function and
+    the latest version is the one that needs to be kept.
+    :param newFunctions: dictionary containing all functions (last version)
     :param oldFunctions: dictionary containing the existing functions.
-    return: dictionary with the merging result (keeping all functions and updating the old ones).
+    return: dictionary with the merging result (keeping all functions and
+            updating the old ones).
     """
     if len(oldFunctions) == 0:
         return newFunctions
     else:
         for fName in newFunctions.keys():
-            if oldFunctions.has_key(fName) and (not newFunctions[fName] == oldFunctions[fName]):
+            if fName in oldFunctions and (not newFunctions[fName] == oldFunctions[fName]):
                 print "WARNING! Function " + fName + " has been redefined with changes (the previous will be deprecated)."
             oldFunctions[fName] = newFunctions[fName]
         return oldFunctions
@@ -445,26 +487,33 @@ def updateFunctions(newFunctions, oldFunctions):
 
 def updateTasks(newTasks, oldTasks):
     """
-    Compare the old tasks against the new ones. This function is essential due to the fact that a jupyter-notebook
-    user may rewrite a task and the latest version is the one that needs to be kept.
+    Compare the old tasks against the new ones. This function is essential due
+    to the fact that a jupyter-notebook user may rewrite a task and the latest
+    version is the one that needs to be kept.
     :param newTask: new Task code
     :param tasks: existing tasks
     return: dictionary with the merging result.
     """
     newTaskName = newTasks.keys()[0]
-    if oldTasks.has_key(newTaskName) and (not newTasks[newTaskName] == oldTasks[newTaskName]):
+    if newTaskName in oldTasks and (not newTasks[newTaskName] == oldTasks[newTaskName]):
         print "WARNING! Task " + newTaskName + " has been redefined with changes (the previous will be deprecated)."
     oldTasks[newTaskName] = newTasks[newTaskName]
     return oldTasks
 
-###  * UPDATE FUNCTIONS
 
-def updateCodeFile(newImports, newGlobals, newClasses, newFunctions, newTasks, filePath):
+# FILE UPDATE FUNCTIONS
+
+def updateCodeFile(newImports, newGlobals, newClasses, newFunctions, newTasks,
+                   filePath):
     """
     Writes the results to the code file used by the workers.
-    :param newImports: Imports
-    :param newCode: Code
-    :param filePath: File location
+    :param newImports: new imports
+    :param newGlobals: new global variables
+    :param newClasses: new classes
+    :param newFunctions: new functions
+    :param newTasks: new tasks
+    :param filePath: File to update.
+    :return:
     """
     file = open(filePath, 'w')
     # Write imports
@@ -499,7 +548,7 @@ def updateCodeFile(newImports, newGlobals, newClasses, newFunctions, newTasks, f
     if len(newFunctions) == 0:
         file.write('\n')
     else:
-        for k,v in newFunctions.iteritems():
+        for k, v in newFunctions.iteritems():
             for line in v:
                 file.write(line)
             file.write('\n')
@@ -510,7 +559,7 @@ def updateCodeFile(newImports, newGlobals, newClasses, newFunctions, newTasks, f
     if len(newTasks) == 0:
         file.write('\n')
     else:
-        for k,v in newTasks.iteritems():
+        for k, v in newTasks.iteritems():
             for line in v:
                 file.write(line)
             file.write('\n')
