@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: srodrig1
 
+"""
 PyCOMPSs Utils: Object properties.
 
 Offers some functions that check properties about objects.
@@ -28,6 +27,7 @@ import ctypes
 import inspect
 import collections
 
+
 def is_module_available(module_name):
     """
     Checks if a module is available in the current Python installation.
@@ -40,6 +40,7 @@ def is_module_available(module_name):
     except:
         return False
 
+
 def is_basic_iterable(obj):
     """
     Checks if an object is a basic iterable.
@@ -82,7 +83,7 @@ def get_object_hierarchy(obj):
     while object_stack:
         current_object = object_stack.pop()
         current_object_id = id(current_object)
-        if not current_object_id in vis:
+        if current_object_id not in vis:
             vis.add(current_object_id)
             yield current_object
             if hasattr(current_object, '__dict__'):
@@ -90,6 +91,7 @@ def get_object_hierarchy(obj):
             elif is_basic_iterable(current_object):
                 map(object_stack.append, iter(current_object))
 
+
 def has_subobjects_of_module(obj, module_name):
     """
     Detects if an object belongs to a module or, at least, has
@@ -109,12 +111,13 @@ def has_subobjects_of_module(obj, module_name):
             return True
     return False
 
+
 def has_numpy_objects(obj):
     """
     Checks if the given object is a numpy object or
     some of its subojects are.
     @param obj: An object
-    @return: Boolean -> True if obj is a numpy objects (or some of 
+    @return: Boolean -> True if obj is a numpy objects (or some of
     its subobjects). False otherwise
     """
     return has_subobjects_of_module(obj, 'numpy')
