@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: srodrig1
 
+"""
 PyCOMPSs Utils: Data serializer/deserializer
 This file implements the main serialization/deserialization functions.
 All serialization/deserialization calls should be made using one of the following functions:
@@ -35,24 +34,35 @@ import types
 import traceback
 import cStringIO as StringIO
 import cPickle as pickle
-from serialization.extendedSupport import copy_generator, pickle_generator, GeneratorSnapshot
-from object_properties import has_numpy_objects
+from serialization.extendedSupport import pickle_generator, convert_to_generator
+from object_properties import object_belongs_to_module
+
 try:
     import dill
 except:
     import cPickle as dill
 
+try:
+    import numpy
+except:
+    import cPickle as numpy
+
+
 class SerializerException(Exception):
     pass
 
+
 def get_serializer_priority(obj=[]):
     """
     Computes the priority of the serializers.
     @param obj: Object to be analysed.
     @return: List -> The serializers sorted by priority in descending order
     """
+    if object_belongs_to_module(obj, 'numpy'):
+        return [numpy, pickle, dill]
     return [pickle, dill]
 
+
 def get_serializers():
     """
     Returns a list with the available serializers in the most common order
@@ -61,11 +71,13 @@ def get_serializers():
     """
     return get_serializer_priority()
 
+
 def serialize_to_handler(obj, handler):
     """
     Serialize an object to a handler.
     @param obj: Object to be serialized.
-    @param file_handler: A handler object. It must implement methods like write, writeline and similar stuff
+    @param file_handler: A handler object. It must implement methods like
+                         write, writeline and similar stuff
     """
     # get the serializer priority
     serializer_priority = get_serializer_priority(obj)
@@ -87,15 +99,20 @@ def serialize_to_handler(obj, handler):
         # general case
         else:
             try:
-                serializer.dump(obj, handler, protocol=serializer.HIGHEST_PROTOCOL)
-                success = True
+                if serializer.__name__ == 'numpy' and type(obj).__module__ == numpy.__name__:
+                    serializer.save(handler, obj)
+                    success = True
+                else:
+                    serializer.dump(obj, handler, protocol=serializer.HIGHEST_PROTOCOL)
+                    success = True
             except:
                 pass
         i += 1
 
     # if ret_value is None then all the serializers have failed
     if not success:
-        raise SerializerException('Cannot serialize object %s'%obj)
+        raise SerializerException('Cannot serialize object %s' % obj)
+
 
 def serialize_to_file(obj, file_name):
     """
@@ -108,6 +125,7 @@ def serialize_to_file(obj, file_name):
     handler.close()
     return file_name
 
+
 def serialize_to_string(obj):
     """
     Serialize an object to a string.
@@ -129,7 +147,7 @@ def deserialize_from_handler(handler):
     @return: The object deserialized.
     """
     # get the most common order of the serializers
-    serializers = get_serializers()
+    serializers = get_serializer_priority(numpy.zeros(1))
     original_position = handler.tell()
     # let's try to deserialize
     for serializer in serializers:
@@ -138,8 +156,8 @@ def deserialize_from_handler(handler):
         try:
             ret = serializer.load(handler)
             # special case: deserialized obj wraps a generator
-            if isinstance(ret, GeneratorSnapshot):
-                ret = copy_generator(ret)[0]
+            if isinstance(ret, tuple) and ret[0] == 'Th3N3xtEl3m3ntIsAG3n3r4t0r':
+                ret = convert_to_generator(ret[1])
             return ret
         except:
             pass
@@ -147,6 +165,7 @@ def deserialize_from_handler(handler):
     # serializers
     raise SerializerException('Cannot deserialize object')
 
+
 def deserialize_from_file(file_name):
     """
     Deserializes the contents in a given file
@@ -158,6 +177,7 @@ def deserialize_from_file(file_name):
     handler.close()
     return ret
 
+
 def deserialize_from_string(serialized_content):
     """
     Deserializes the contents in a given string
