@@ -1,5 +1,5 @@
 #
-#  Copyright 2.02-2017 Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -13,11 +13,8 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #
-"""
-@author: fconejer
-@author: cramonco
-@author: srodrig1
 
+"""
 PyCOMPSs Persistent Worker
 ===========================
     This file contains the worker code.
@@ -28,13 +25,17 @@ import os
 import signal
 import sys
 import traceback
-import thread_affinity
 from exceptions import ValueError
-from multiprocessing import Process, Queue, Pipe
+from multiprocessing import Process
+from multiprocessing import Queue
 import thread_affinity
 
-from pycompss.api.parameter import Type, JAVA_MAX_INT, JAVA_MIN_INT
-from pycompss.util.serializer import serialize_to_file, deserialize_from_file, deserialize_from_string, SerializerException
+from pycompss.api.parameter import TYPE
+from pycompss.api.parameter import JAVA_MIN_INT, JAVA_MAX_INT
+from pycompss.util.serializer import serialize_to_file
+from pycompss.util.serializer import deserialize_from_file
+from pycompss.util.serializer import deserialize_from_string
+from pycompss.util.serializer import SerializerException
 from pycompss.util.logs import init_logging_worker
 
 SYNC_EVENTS = 8000666
@@ -56,9 +57,8 @@ tracing = False
 debug = True
 processes = []
 
-#if sys.version_info >= (2, 7):
-#    import importlib
-
+# if sys.version_info >= (2, 7):
+#     import importlib
 
 
 #####################
@@ -68,7 +68,7 @@ INIT = "init"  # -- worker.py debug tracing #thr pipes_CMD pipes_RESULT
 EXECUTE_TASK_TAG = "task"  # -- "task" taskId jobOut jobErr task_params
 END_TASK_TAG = "endTask"  # -- "endTask" taskId endStatus
 QUIT_TAG = "quit"  # -- "quit"
-USE_CACHE = False
+
 
 ######################
 #  Processes body
@@ -93,6 +93,13 @@ def worker(queue, process_name, input_pipe, output_pipe, storage_conf):
     level = logger.getEffectiveLevel()
     formatter = logging.Formatter(handler.formatter._fmt)
 
+    if storage_conf != 'null':
+        try:
+            from storage.api import initWorkerPostFork as initStorageAtWorkerPostFork
+            initStorageAtWorkerPostFork()
+        except:
+            logger.info("[PYTHON WORKER] Could not find initWorkerPostFork storage call. Ignoring it.")
+
     # TRACING
     # if tracing:
     #     pyextrae.eventandcounters(TASK_EVENTS, 0)
@@ -101,12 +108,6 @@ def worker(queue, process_name, input_pipe, output_pipe, storage_conf):
     stdout = sys.stdout
     stderr = sys.stderr
 
-
-    local_cache = None
-    if USE_CACHE:
-        from persistent_cache import Cache
-        local_cache = Cache(size_limit = 1024**3)
-
     logger.debug("[PYTHON WORKER] Starting process " + str(process_name))
     while alive:
         with open(input_pipe, 'r', 0) as in_pipe:
@@ -130,13 +131,26 @@ def worker(queue, process_name, input_pipe, output_pipe, storage_conf):
                             job_id = line[1]
                             job_out = line[2]
                             job_err = line[3]
-                            # line[4] = <boolean> = ?
-                            # line[5] = <integer> = ?
-                            # line[6] = <boolean> = ?
-                            # line[7] = null      = ?
+                            # line[4] = <boolean> = tracing
+                            # line[5] = <integer> = task id
+                            # line[6] = <boolean> = debug
+                            # line[7] = <string>  = storage conf.
                             # line[8] = <string>  = operation type (e.g. METHOD)
+                            # line[9] = <string>  = module
+                            # line[10]= <string>  = method
+                            # line[11]= <integer> = Number of slaves (worker nodes) == #nodes
+                            # <<list of slave nodes>>
+                            # line[11 + #nodes] = <integer> = computing units
+                            # line[12 + #nodes] = <boolean> = has target
+                            # line[13 + #nodes] = <string>  = has return (always 'null')
+                            # line[14 + #nodes] = <integer> = Number of parameters
+                            # <<list of parameters>>
+                            #       !---> type, stream, prefix , value
 
-                            # Swap logger from stream handler to file handler.
+                            logger.debug("[PYTHON WORKER %s] Received task." % str(process_name))
+                            logger.debug("[PYTHON WORKER %s] - TASK CMD: %s" % (str(process_name), str(line)))
+
+                            # Swap logger from stream handler to file handler.   #### TODO: FIX LOGGER!
                             logger.removeHandler(logger.handlers[0])
                             out_file_handler = logging.FileHandler(job_out)
                             out_file_handler.setLevel(level)
@@ -154,21 +168,47 @@ def worker(queue, process_name, input_pipe, output_pipe, storage_conf):
                                 err = open(job_err, 'w')
                                 sys.stdout = out
                                 sys.stderr = err
-                                exitvalue = execute_task(process_name, storage_conf, line[9:], local_cache)
+                                exitvalue, newTypes, newValues = execute_task(process_name, storage_conf, line[9:])
                                 sys.stdout = stdout
                                 sys.stderr = stderr
+                                sys.stdout.flush()
+                                sys.stderr.flush()
                                 out.close()
                                 err.close()
 
-                                # endTask jobId exitValue
-                                message = END_TASK_TAG + " " + str(job_id) + " " + str(exitvalue) + "\n"
-                                logger.debug("[PYTHON WORKER %s] - Pipe %s END TASK MESSAGE: %s" %(str(process_name),
-                                                                                                   str(output_pipe),
-                                                                                                   str(message)))
+                                if exitvalue == 0:
+                                    # Task has finished without exceptions
+                                    # endTask jobId exitValue message
+                                    params = buildReturnParamsMessage(line[9:], newTypes, newValues)
+                                    message = END_TASK_TAG + " " + str(job_id) \
+                                                           + " " + str(exitvalue) \
+                                                           + " " + str(params) + "\n"
+                                else:
+                                    # An exception has been raised in task
+                                    message = END_TASK_TAG + " " + str(job_id) \
+                                              + " " + str(exitvalue) + "\n"
+
+                                logger.debug("[PYTHON WORKER %s] - Pipe %s END TASK MESSAGE: %s" % (str(process_name),
+                                                                                                    str(output_pipe),
+                                                                                                    str(message)))
+                                # The return message is:
+                                #
+                                # TaskResult ==> jobId exitValue List<Object>
+                                #
+                                # Where List<Object> has D length:
+                                # D = #parameters + (hasTarget ? 1 : 0) + (hasReturn ? 1 : 0)
+                                # And contains:
+                                # - Null if it NOT a PSCO
+                                # - PSCOId (String) if is a PSCO
+                                #
+                                # This is sent through the pipe with the endTask message.
+                                # If the task had an object or file as parameter and the worker returns the id,
+                                # the runtime can change the type (and locations) to a EXTERNAL_OBJ_T.
+
                                 with open(output_pipe, 'w+') as out_pipe:
                                     out_pipe.write(message)
                             except Exception, e:
-                                logger.exception("[PYTHON WORKER %s] Exception %s" %(str(process_name), str(e)))
+                                logger.exception("[PYTHON WORKER %s] Exception %s" % (str(process_name), str(e)))
                                 queue.put("EXCEPTION")
 
                             # Restore logger
@@ -190,10 +230,40 @@ def worker(queue, process_name, input_pipe, output_pipe, storage_conf):
     sys.stderr.flush()
     print("[PYTHON WORKER] Exiting process ", process_name)
 
+
+def buildReturnParamsMessage(params, types, values):
+    assert len(types) == len(values), 'Inconsistent state: return type-value length mismatch for return message.'
+
+    # Analize the input parameters to get has_target and has_return
+    # More information that requested can be gathered and returned in the return message if necessary.
+    numSlaves = int(params[2])
+    argPosition = 3 + numSlaves
+    args = params[argPosition + 1:]
+    has_target = args[0]
+    if has_target == 'false':
+        hasTarget = False
+    else:
+        hasTarget = True
+    return_type = args[1]
+    if return_type == 'null':
+        hasReturn = False
+    else:
+        hasReturn = True
+
+    pairs = zip(types, values)
+    num_params = len(pairs)
+    params = ''
+    for p in pairs:
+        params = params + str(p[0]) + ' ' + str(p[1]) + ' '
+    totalParams = num_params + (1 if hasTarget else 0) + (1 if hasReturn else 0)
+    message = str(totalParams) + ' ' + params
+    return message
+
+
 #####################################
 # Execute Task Method - Task handler
 #####################################
-def execute_task(process_name, storage_conf, params, local_cache):
+def execute_task(process_name, storage_conf, params):
     """
     ExecuteTask main method
     """
@@ -207,11 +277,10 @@ def execute_task(process_name, storage_conf, params, local_cache):
         from storage.api import getByID
         from storage.api import TaskContext
 
-    # COMPSs keywords for tasks (ie: tracing, cache data structures...)
+    # COMPSs keywords for tasks (ie: tracing, process name...)
     compss_kwargs = {
-        'compss_tracing' : tracing,
+        'compss_tracing': tracing,
         'compss_process_name': process_name,
-        'compss_local_cache': local_cache,
         'compss_storage_conf': storage_conf
     }
 
@@ -234,109 +303,29 @@ def execute_task(process_name, storage_conf, params, local_cache):
 
     args = args[3:]
 
-    logger.debug("[PYTHON WORKER %s] Storage conf: %s" % (str(process_name), str(storage_conf)))
-    logger.debug("[PYTHON WORKER %s] Params: %s" % (str(process_name), str(params)))
-    logger.debug("[PYTHON WORKER %s] Path: %s" % (str(process_name), str(path)))
-    logger.debug("[PYTHON WORKER %s] Method name: %s" % (str(process_name), str(method_name)))
-    logger.debug("[PYTHON WORKER %s] Num slaves: %s" % (str(process_name), str(numSlaves)))
-    logger.debug("[PYTHON WORKER %s] Slaves: %s" % (str(process_name), str(slaves)))
-    logger.debug("[PYTHON WORKER %s] Cus: %s" % (str(process_name), str(cus)))
-    logger.debug("[PYTHON WORKER %s] Has target: %s" % (str(process_name), str(has_target)))
-    logger.debug("[PYTHON WORKER %s] Num Params: %s" % (str(process_name), str(num_params)))
-    logger.debug("[PYTHON WORKER %s] Args: %s" % (str(process_name), str(args)))
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug("[PYTHON WORKER %s] Storage conf: %s" % (str(process_name), str(storage_conf)))
+        logger.debug("[PYTHON WORKER %s] Params: %s" % (str(process_name), str(params)))
+        logger.debug("[PYTHON WORKER %s] Path: %s" % (str(process_name), str(path)))
+        logger.debug("[PYTHON WORKER %s] Method name: %s" % (str(process_name), str(method_name)))
+        logger.debug("[PYTHON WORKER %s] Num slaves: %s" % (str(process_name), str(numSlaves)))
+        logger.debug("[PYTHON WORKER %s] Slaves: %s" % (str(process_name), str(slaves)))
+        logger.debug("[PYTHON WORKER %s] Cus: %s" % (str(process_name), str(cus)))
+        logger.debug("[PYTHON WORKER %s] Has target: %s" % (str(process_name), str(has_target)))
+        logger.debug("[PYTHON WORKER %s] Num Params: %s" % (str(process_name), str(num_params)))
+        logger.debug("[PYTHON WORKER %s] Args: %r" % (str(process_name), args))
 
-    pos = 0
-
-    values = []
-    types = []
-    streams = []
-    prefixes = []
-
-    #if tracing:
-    #    pyextrae.event(TASK_EVENTS, 0)
-    #    pyextrae.event(TASK_EVENTS, PARAMETER_PROCESSING)
+    # if tracing:
+    #     pyextrae.event(TASK_EVENTS, 0)
+    #     pyextrae.event(TASK_EVENTS, PARAMETER_PROCESSING)
 
     # Get all parameter values
     logger.debug("[PYTHON WORKER %s] Processing parameters:" % process_name)
-    for i in range(0, num_params):
-        pType = int(args[pos])
-        pStream = int(args[pos + 1])
-        pPrefix = args[pos + 2]
-        pValue = args[pos + 3]
+    values, types, streams, prefixes = get_input_params(num_params, logger, args, process_name, persistent_storage)
 
-        logger.debug("[PYTHON WORKER %s] Parameter : %s" % (process_name, str(i)))
-        logger.debug("[PYTHON WORKER %s] \t * Type : %s" % (process_name, str(pType)))
-        logger.debug("[PYTHON WORKER %s] \t * Stream : %s" % (process_name, str(pStream)))
-        logger.debug("[PYTHON WORKER %s] \t * Prefix : %s" % (process_name, str(pPrefix)))
-        logger.debug("[PYTHON WORKER %s] \t * Value: %s" % (process_name, str(pValue)))
-
-        types.append(pType)
-        streams.append(pStream)
-        prefixes.append(pPrefix)
-
-        if pType == Type.FILE:
-            ## check if it is a persistent object --- TO REMOVE THESE LINES
-            #if 'getID' in dir(pValue) and pValue.getID() is not None:
-            #    po = getByID(pValue.getID())
-            #    values.append(po)
-            #else:
-            #    values.append(pValue)
-            values.append(pValue)
-        elif pType == Type.EXTERNAL_PSCO:
-            po = getByID(pValue)
-            values.append(po)
-            pos += 1  # Skip info about direction (R, W)
-        elif pType == Type.STRING:
-            num_substrings = int(pValue)
-            aux = ''
-            first_substring = True
-            for j in range(4, num_substrings + 4):
-                if not first_substring:
-                    aux += ' '
-                first_substring = False
-                aux += args[pos + j]
-            #######
-            # Check if the string is really an object
-            # Required in order to recover objects passed as parameters.
-            # - Option object_conversion
-            real_value = aux
-            try:
-                # try to recover the real object
-                aux = deserialize_from_string(aux)
-            except (SerializerException, ValueError, EOFError):
-                # was not an object
-                aux = real_value
-            #######
-            values.append(aux)
-            logger.debug("[PYTHON WORKER %s] \t * Final Value: %s" % (process_name, str(aux)))
-            pos += num_substrings
-        elif pType == Type.INT:
-            values.append(int(pValue))
-        elif pType == Type.LONG:
-            l = long(pValue)
-            if l > JAVA_MAX_INT or l < JAVA_MIN_INT:
-                # A Python int was converted to a Java long to prevent overflow
-                # We are sure we will not overflow Python int, otherwise this
-                # would have been passed as a serialized object.
-                l = int(l)
-            values.append(l)
-        elif pType == Type.DOUBLE:
-            values.append(float(pValue))
-        elif pType == Type.BOOLEAN:
-            if pValue == 'true':
-                values.append(True)
-            else:
-                values.append(False)
-        # elif (pType == Type.OBJECT):
-        #    pass
-        else:
-            logger.fatal("[PYTHON WORKER %s] Invalid type (%d) for parameter %d" % (process_name, pType, i))
-            exit(1)
-        pos += 4
-
-    #if tracing:
-    #    pyextrae.event(TASK_EVENTS, 0)
-    #    pyextrae.event(TASK_EVENTS, LOGGING)
+    # if tracing:
+    #     pyextrae.event(TASK_EVENTS, 0)
+    #     pyextrae.event(TASK_EVENTS, LOGGING)
 
     if logger.isEnabledFor(logging.DEBUG):
         logger.debug("[PYTHON WORKER %s] RUN TASK with arguments: " % process_name)
@@ -346,17 +335,20 @@ def execute_task(process_name, storage_conf, params, local_cache):
         logger.debug("[PYTHON WORKER %s] \t- # parameters: %s" % (process_name, str(num_params)))
         logger.debug("[PYTHON WORKER %s] \t- Values:" % process_name)
         for v in values:
-            logger.debug("[PYTHON WORKER %s] \t\t %s" % (process_name, str(v)))
+            logger.debug("[PYTHON WORKER %s] \t\t %r" % (process_name, v))
         logger.debug("[PYTHON WORKER %s] \t- COMPSs types:" % process_name)
         for t in types:
             logger.debug("[PYTHON WORKER %s] \t\t %s" % (process_name, str(t)))
 
-    #if tracing:
-    #    pyextrae.event(TASK_EVENTS, 0)
-    #    pyextrae.event(TASK_EVENTS, MODULES_IMPORT)
+    # if tracing:
+    #     pyextrae.event(TASK_EVENTS, 0)
+    #     pyextrae.event(TASK_EVENTS, MODULES_IMPORT)
 
     import_error = False
 
+    newTypes = []
+    newValues = []
+
     try:
         # Try to import the module (for functions)
         logger.debug("[PYTHON WORKER %s] Trying to import the user module." % process_name)
@@ -369,21 +361,13 @@ def execute_task(process_name, storage_conf, params, local_cache):
             logger.debug("[PYTHON WORKER %s] Module successfully loaded (Python version < 2.7" % process_name)
 
         def task_execution_1():
-            # if tracing:
-            #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-            #    pyextrae.eventandcounters(TASK_EVENTS, TASK_EXECUTION)
-            logger.debug("[PYTHON WORKER %s] Starting task execution" % process_name)
-            getattr(module, method_name)(*values, compss_types=types, **compss_kwargs)
-            logger.debug("[PYTHON WORKER %s] Finished task execution" % process_name)
-            # if tracing:
-            #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-            #    pyextrae.eventandcounters(TASK_EVENTS, WORKER_END)
+            return task_execution(logger, process_name, module, method_name, types, values, compss_kwargs)
 
         if persistent_storage:
             with TaskContext(logger, values, config_file_path=storage_conf):
-                task_execution_1()
+                newTypes, newValues = task_execution_1()
         else:
-            task_execution_1()
+            newTypes, newValues = task_execution_1()
 
     # ==========================================================================
     except AttributeError:
@@ -393,7 +377,9 @@ def execute_task(process_name, storage_conf, params, local_cache):
         logger.exception("[PYTHON WORKER %s] WORKER EXCEPTION - Attribute Error Exception" % process_name)
         logger.exception(''.join(line for line in lines))
         logger.exception("[PYTHON WORKER %s] Check that all parameters have been defined with an absolute import path (even if in the same file)" % process_name)
-        exit(1)
+        # If exception is raised during the task execution, newTypes and
+        # newValues are empty
+        return 1, newTypes, newValues
     # ==========================================================================
     except ImportError:
         import_error = True
@@ -403,7 +389,9 @@ def execute_task(process_name, storage_conf, params, local_cache):
         lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
         logger.exception("[PYTHON WORKER %s] WORKER EXCEPTION" % process_name)
         logger.exception(''.join(line for line in lines))
-        return 1
+        # If exception is raised during the task execution, newTypes and
+        # newValues are empty
+        return 1, newTypes, newValues
 
     if import_error:
         logger.debug("[PYTHON WORKER %s] Could not import the module. Reason: Method in class." % process_name)
@@ -428,48 +416,32 @@ def execute_task(process_name, storage_conf, params, local_cache):
             logger.debug("[PYTHON WORKER %s] Processing callee, a hidden object of %s in file %s" % (process_name, file_name, type(obj)))
             values.insert(0, obj)
             types.pop()
-            types.insert(0, Type.OBJECT)
+            types.insert(0, TYPE.OBJECT)
 
             def task_execution_2():
-                # if tracing:
-                #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-                #    pyextrae.eventandcounters(TASK_EVENTS, TASK_EXECUTION)
-                logger.debug("[PYTHON WORKER %s] Starting task execution")
-                getattr(klass, method_name)(*values, compss_types=types, **compss_kwargs)
-                logger.debug("[PYTHON WORKER %s] Finished task execution")
-                # if tracing:
-                #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-                #    pyextrae.eventandcounters(TASK_EVENTS, WORKER_END)
+                return task_execution(logger, process_name, klass, method_name, types, values, compss_kwargs)
 
             if persistent_storage:
                 with TaskContext(logger, values, config_file_path=storage_conf):
-                    task_execution_2()
+                    newTypes, newValues = task_execution_2()
             else:
-                task_execution_2()
+                newTypes, newValues = task_execution_2()
 
             logger.debug("[PYTHON WORKER %s] Serializing self to file." % process_name)
-            logger.debug("[PYTHON WORKER %s] Obj: %s" % (process_name,str(obj)))
+            logger.debug("[PYTHON WORKER %s] Obj: %r" % (process_name, obj))
             serialize_to_file(obj, file_name)
         else:
             # Class method - class is not included in values (e.g. values = [7])
             types.insert(0, None)  # class must be first type
 
             def task_execution_3():
-                # if tracing:
-                #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-                #    pyextrae.eventandcounters(TASK_EVENTS, TASK_EXECUTION)
-                logger.debug("[PYTHON WORKER %s] Starting task execution")
-                getattr(klass, method_name)(*values, compss_types=types, **compss_kwargs)
-                logger.debug("[PYTHON WORKER %s] Finished task execution")
-                # if tracing:
-                #    pyextrae.eventandcounters(TASK_EVENTS, 0)
-                #    pyextrae.eventandcounters(TASK_EVENTS, WORKER_END)
+                return task_execution(logger, process_name, klass, method_name, types, values, compss_kwargs)
 
             if persistent_storage:
                 with TaskContext(logger, values, config_file_path=storage_conf):
-                    task_execution_3()
+                    newTypes, newValues = task_execution_3()
             else:
-                task_execution_3()
+                newTypes, newValues = task_execution_3()
 
     # EVERYTHING OK
     logger.debug("[PYTHON WORKER %s] End task execution. Status: Ok" % process_name)
@@ -477,9 +449,128 @@ def execute_task(process_name, storage_conf, params, local_cache):
     # if tracing:
     #     pyextrae.eventandcounters(TASK_EVENTS, 0)
 
-    return 0
+    return 0, newTypes, newValues   # Exit code, updated params
 
 
+def get_input_params(num_params, logger, args, process_name, persistent_storage):
+    if persistent_storage:
+        from storage.api import getByID
+        from storage.api import TaskContext
+    pos = 0
+    values = []
+    types = []
+    streams = []
+    prefixes = []
+    for i in range(0, num_params):
+        pType = int(args[pos])
+        pStream = int(args[pos + 1])
+        pPrefix = args[pos + 2]
+        pValue = args[pos + 3]
+
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug("[PYTHON WORKER %s] Parameter : %s" % (process_name, str(i)))
+            logger.debug("[PYTHON WORKER %s] \t * Type : %s" % (process_name, str(pType)))
+            logger.debug("[PYTHON WORKER %s] \t * Stream : %s" % (process_name, str(pStream)))
+            logger.debug("[PYTHON WORKER %s] \t * Prefix : %s" % (process_name, str(pPrefix)))
+            logger.debug("[PYTHON WORKER %s] \t * Value: %r" % (process_name, pValue))
+
+        types.append(pType)
+        streams.append(pStream)
+        prefixes.append(pPrefix)
+
+        if pType == TYPE.FILE:
+            values.append(pValue)
+        elif pType == TYPE.EXTERNAL_PSCO:
+            po = getByID(pValue)
+            values.append(po)
+            pos += 1  # Skip info about direction (R, W)
+        elif pType == TYPE.STRING:
+            num_substrings = int(pValue)
+            aux = ''
+            first_substring = True
+            for j in range(4, num_substrings + 4):
+                if not first_substring:
+                    aux += ' '
+                first_substring = False
+                aux += args[pos + j]
+            #######
+            # Check if the string is really an object
+            # Required in order to recover objects passed as parameters.
+            # - Option object_conversion
+            real_value = aux
+            try:
+                # try to recover the real object
+                aux = deserialize_from_string(aux.decode('string_escape'))
+            except (SerializerException, ValueError, EOFError):
+                # was not an object
+                aux = real_value
+            #######
+            values.append(aux)
+            logger.debug("[PYTHON WORKER %s] \t * Final Value: %s" % (process_name, str(aux)))
+            pos += num_substrings
+        elif pType == TYPE.INT:
+            values.append(int(pValue))
+        elif pType == TYPE.LONG:
+            l = long(pValue)
+            if l > JAVA_MAX_INT or l < JAVA_MIN_INT:
+                # A Python int was converted to a Java long to prevent overflow
+                # We are sure we will not overflow Python int, otherwise this
+                # would have been passed as a serialized object.
+                l = int(l)
+            values.append(l)
+        elif pType == TYPE.DOUBLE:
+            values.append(float(pValue))
+        elif pType == TYPE.BOOLEAN:
+            if pValue == 'true':
+                values.append(True)
+            else:
+                values.append(False)
+        # elif (pType == TYPE.OBJECT):
+        #    pass
+        else:
+            logger.fatal("[PYTHON WORKER %s] Invalid type (%d) for parameter %d" % (process_name, pType, i))
+            exit(1)
+        pos += 4
+    return values, types, streams, prefixes
+
+
+def task_execution(logger, process_name, module, method_name, types, values, compss_kwargs):
+    # if tracing:
+    #    pyextrae.eventandcounters(TASK_EVENTS, 0)
+    #    pyextrae.eventandcounters(TASK_EVENTS, TASK_EXECUTION)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug("[PYTHON WORKER %s] Starting task execution" % process_name)
+        logger.debug("[PYTHON WORKER %s] Types : %s " % (process_name, str(types)))
+        logger.debug("[PYTHON WORKER %s] Values: %s " % (process_name, str(values)))
+
+    # WARNING: the following call will not work if a user decorator overrides the return of the task decorator.
+    # newTypes, newValues = getattr(module, method_name)(*values, compss_types=types, **compss_kwargs)
+    # If the @task is decorated with a user decorator, may include more return values, and consequently,
+    # the newTypes and newValues will be within a tuple at position 0.
+    # Force users that use decorators on top of @task to return the task results first.
+    # This is tested with the timeit decorator in test 19.
+    taskOutput = getattr(module, method_name)(*values, compss_types=types, **compss_kwargs)
+
+    if isinstance(taskOutput[0], tuple):  # Weak but effective way to check it without doing inspect.
+        # Another decorator has added another return thing.
+        # TODO: Should we consider here to create a list with all elements and serialize it to a file with the real task output plus the decorator results? == taskOutput[1:]
+        # TODO: Currently, the extra result is ignored.
+        newTypes = taskOutput[0][0]
+        newValues = taskOutput[0][1]
+    else:
+        # The taskOutput is composed by the newTypes and newValues returned by the task decorator.
+        newTypes = taskOutput[0]
+        newValues = taskOutput[1]
+
+    if logger.isEnabledFor(logging.DEBUG):
+        # The types may change (e.g. if the user does a makePersistent within the task)
+        logger.debug("[PYTHON WORKER %s] Return Types : %s " % (process_name, str(newTypes)))
+        logger.debug("[PYTHON WORKER %s] Return Values: %s " % (process_name, str(newValues)))
+        logger.debug("[PYTHON WORKER %s] Finished task execution" % process_name)
+    # if tracing:
+    #    pyextrae.eventandcounters(TASK_EVENTS, 0)
+    #    pyextrae.eventandcounters(TASK_EVENTS, WORKER_END)
+    return newTypes, newValues
 
 
 def shutdown_handler(signal, frame):
