@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/addr2info.c $
- | @last_commit: $Date: 2015-04-27 12:29:08 +0200 (lun, 27 abr 2015) $
- | @version:     $Revision: 3279 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: addr2info.c 3279 2015-04-27 10:29:08Z harald $";
-
 #ifdef HAVE_STDIO_H
 # include <stdio.h>
 #endif
@@ -46,7 +39,7 @@ static char UNUSED rcsid[] = "$Id: addr2info.c 3279 2015-04-27 10:29:08Z harald
 # include <libgen.h>
 #endif
 
-//#define DEBUG
+// #define DEBUG
 
 #define SHORT_STRING_PREFIX 8
 #define SHORT_STRING_SUFFIX 8
@@ -76,12 +69,20 @@ static void AddressTable_Initialize (void);
 static int  AddressTable_Insert (UINT64 address, int event_type,
 	char *module, char *funcname, char *filename, int line);
 #if defined(HAVE_BFD)
+static int AddressTable_Insert_MemReference (int addr_type,
+	const char *module, const char *staticname, const char *filename, int line);
+#endif
+#if defined(HAVE_BFD)
 static void Translate_Address (UINT64 address, unsigned ptask, unsigned task,
 	char **module, char ** funcname, char ** filename, int * line);
+static void Translate_Address_Data (UINT64 address, unsigned ptask, unsigned task,
+	char **symbol);
 #endif
 
-struct address_table  * AddressTable  [COUNT_ADDRESS_TYPES]; /* Addresses translation table     */
-struct function_table * FunctionTable [COUNT_ADDRESS_TYPES]; /* Function name translation table */
+static struct address_table  * AddressTable  [COUNT_ADDRESS_TYPES]; /* Addresses translation table     */
+static struct function_table * FunctionTable [COUNT_ADDRESS_TYPES]; /* Function name translation table */
+
+static struct address_object_table_st AddressObjectInfo;
 
 /* Addresses will be translated into function, file and line if set to TRUE */
 static int Tables_Initialized = FALSE;
@@ -211,6 +212,11 @@ void Address2Info_Initialize (char * binary)
 
 	if (binary != NULL)
 		BFDmanager_loadDefaultBinary (binary);
+
+	AddressTable_Insert_MemReference (MEM_REFERENCE_DYNAMIC, "", "",
+	  ADDR_UNRESOLVED, 0);
+	AddressTable_Insert_MemReference (MEM_REFERENCE_STATIC, "", ADDR_UNRESOLVED,
+	  "", 0);
 #else
 	UNREFERENCED_PARAMETER(binary);
 #endif
@@ -253,6 +259,10 @@ static void AddressTable_Initialize (void)
 		FunctionTable[type]->address_id = NULL;
 		FunctionTable[type]->num_functions = 0;
 	}
+
+	AddressObjectInfo.objects = NULL;
+	AddressObjectInfo.num_objects = 0;
+
 	Tables_Initialized = TRUE;
 }
 
@@ -274,6 +284,101 @@ void Address2Info_AddSymbol (UINT64 address, int addr_type, char * funcname,
 		AddressTable_Insert (address, addr_type, NULL, strdup(funcname), strdup(filename), line);
 }
 
+/** Address2Info_Translate_MemReference
+ *
+ * 
+ * Si l'adreca demanada no estava ja a la taula d'adreces, la tradueix pel nom
+ * de la funcio i el numero de linea de codi corresponents, li assigna un nou
+ * identificador i afegeix una nova entrada a la taula
+ * 
+ * @param address
+ * @param query
+ *
+ * @return
+ */
+UINT64 Address2Info_Translate_MemReference (unsigned ptask, unsigned task, UINT64 address,
+	int query, UINT64 *calleraddresses)
+{
+#if !defined(HAVE_BFD)
+	UNREFERENCED_PARAMETER(ptask);
+	UNREFERENCED_PARAMETER(task);
+	UNREFERENCED_PARAMETER(address);
+	UNREFERENCED_PARAMETER(query);
+	UNREFERENCED_PARAMETER(calleraddresses);
+
+	return address;
+#else
+
+#if defined(DEBUG)
+	fprintf (stderr, PACKAGE_NAME": Address2Info_TranslateMemreference (%u, %u, %lx, %d);\n",
+	  ptask, task, address, query);
+#endif
+
+	if (query == MEM_REFERENCE_DYNAMIC)
+	{
+		int line;
+		char * sname;
+		char * filename;
+		char * module;
+		char buffer[2048];
+		char tmp[1024];
+		int i;
+
+		snprintf (buffer, sizeof(buffer), "");
+
+		/* Trim head and tail for callers that can't be translated */
+		for (i = 0; i < MAX_CALLERS; i++)
+			if (calleraddresses[i] != 0)
+			{
+				Translate_Address (calleraddresses[i], ptask, task, &module,
+				  &sname, &filename, &line);
+				if (!strcmp (filename, ADDR_UNRESOLVED) || !strcmp (filename, ADDR_NOT_FOUND))
+					calleraddresses[i] = 0;
+				else
+					break;
+			}
+
+		for (i = MAX_CALLERS-1; i >= 0; i--)
+			if (calleraddresses[i] != 0)
+			{
+				Translate_Address (calleraddresses[i], ptask, task, &module,
+				  &sname, &filename, &line);
+				if (!strcmp (filename, ADDR_UNRESOLVED) || !strcmp (filename, ADDR_NOT_FOUND))
+					calleraddresses[i] = 0;
+				else
+					break;
+			}
+
+		for (i = 0; i < MAX_CALLERS; i++)
+			if (calleraddresses[i] != 0)
+			{
+				Translate_Address (calleraddresses[i], ptask, task, &module,
+				  &sname, &filename, &line);
+				if (strlen(buffer) > 0)
+					snprintf (tmp, sizeof(tmp), " > %s:%d", filename, line);
+				else
+					snprintf (tmp, sizeof(tmp), "%s:%d", filename, line);
+				strncat (buffer, tmp, sizeof(buffer));
+			}
+
+		return 1+AddressTable_Insert_MemReference (query, module, "",
+		  strdup(buffer), 0);
+
+	}
+	else if (query == MEM_REFERENCE_STATIC)
+	{
+		char *varname;
+
+		Translate_Address_Data (address, ptask, task, &varname);
+
+		return 1+AddressTable_Insert_MemReference (query, "", varname, "", 0);
+	}
+	else
+		return address;
+
+#endif
+}
+
 /** Address2Info_Translate
  *
  * 
@@ -307,6 +412,11 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 	UINT64 result;
 	int found;
 
+#if defined(DEBUG)
+	fprintf (stderr, PACKAGE_NAME": Address2Info_Translate (%u, %u, %lx, %d, %d);\n",
+	  ptask, task, address, query, uniqueID);
+#endif
+
 /* address es la direccion de retorno por donde continuara ejecutandose el
  * codigo despues de cada CALL a una rutina MPI. En arquitecturas x86, despues
  * de la llamada se desapilan los parametros y se recogen los resultados. Asi
@@ -420,13 +530,12 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 		char * filename;
 		char * module;
 
-		/* Translate address into function, file and number line */
 		Translate_Address (caller_address, ptask, task, &module, &funcname, &filename, &line);
 
 		/* Samples can be taken anywhere in the code. It can happen that two
 		   samples at the different addresses refer to the same combination of
 		   function/file/line... We search again if this combination has been
-		   already translated */
+		   already translated. */
 		if (ADDR2SAMPLE_FUNCTION == query || ADDR2SAMPLE_LINE == query)
 			for (i = 0; i < AddressTable[addr_type]->num_addresses; i++)
 				if (AddressTable[addr_type]->address[i].line == line &&
@@ -444,13 +553,15 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 			line_id = UNRESOLVED_ID;
 			already_translated = TRUE;
 		}
-		else if (!strcmp (ADDR_UNRESOLVED, funcname) || !strcmp (ADDR_UNRESOLVED, filename))
+		else if (!strcmp (ADDR_UNRESOLVED, funcname) ||
+		         !strcmp (ADDR_UNRESOLVED, filename))
 		{
 			function_id = UNRESOLVED_ID;
 			line_id = UNRESOLVED_ID;
 			already_translated = TRUE;
 		}
-		else if (!strcmp(ADDR_NOT_FOUND, funcname) || !strcmp (ADDR_NOT_FOUND, filename))
+		else if (!strcmp(ADDR_NOT_FOUND, funcname) ||
+		         !strcmp (ADDR_NOT_FOUND, filename))
 		{
 			function_id = NOT_FOUND_ID;
 			line_id = NOT_FOUND_ID;
@@ -460,7 +571,8 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 		/* Again, if not found, insert into the translation table */
 		if (!already_translated)
 		{
-			line_id = AddressTable_Insert (address, addr_type, module, funcname, filename, line);
+			line_id = AddressTable_Insert (address, addr_type, module, funcname,
+			  filename, line);
 			function_id = AddressTable[addr_type]->address[line_id].function_id;
 		}
 	}
@@ -485,6 +597,8 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 		case ADDR2SAMPLE_LINE:
 		case ADDR2OMP_LINE:
 		case ADDR2MPI_LINE:
+		case MEM_REFERENCE_DYNAMIC:
+		case MEM_REFERENCE_STATIC:
 			result = line_id + 1;
 			break;
 	}
@@ -493,6 +607,74 @@ UINT64 Address2Info_Translate (unsigned ptask, unsigned task, UINT64 address,
 #endif /* HAVE_BFD */
 }
 
+/** AddressTable_Insert_MemReference
+ *
+ * @param address
+ * @param funcname
+ * @param filename
+ * @param line
+ * 
+ * @return 
+ */
+#if defined(HAVE_BFD)
+static int AddressTable_Insert_MemReference (int addr_type,
+	const char *module, const char *staticname, const char *filename, int line) 
+{
+	int i; 
+	int found = FALSE;
+
+#if defined(DEBUG) 
+	fprintf (stderr, PACKAGE_NAME": AddressTable_Insert_MemReference (%d, %s, %s, %s, %d)\n",
+	  addr_type, module, staticname, filename, line);
+#endif
+
+	int nObjects = AddressObjectInfo.num_objects;
+
+	/* Search if it exists */
+	for (i = 0; i < nObjects; i++)
+	{
+		struct address_object_info_st * object = &(AddressObjectInfo.objects[i]);
+		if (addr_type == MEM_REFERENCE_STATIC && object->is_static)
+			found = !strcmp (staticname, object->name);
+		else if (addr_type == MEM_REFERENCE_DYNAMIC && !object->is_static)
+			found = !strcmp (filename, object->file_name);
+		if (found)
+		{
+#if defined(DEBUG)
+			fprintf (stderr, PACKAGE_NAME": AddressTable_Insert_MemReference (%d, %s, %s, %s, %d) -> %d\n",
+			  addr_type, module, staticname, filename, line, i);
+#endif
+			return i;
+		}
+	}
+
+	/* If we're here, we haven't find it! */
+	AddressObjectInfo.objects = (struct address_object_info_st*) realloc (
+	  AddressObjectInfo.objects,
+	  (AddressObjectInfo.num_objects+1)*sizeof(struct address_object_info_st));
+	if (NULL == AddressObjectInfo.objects)
+	{
+		fprintf (stderr, "mpi2prv: Error! Cannot reallocate memory for memory object identifiers\n");
+		exit (-1);
+	}
+
+	i = AddressObjectInfo.num_objects;
+	AddressObjectInfo.objects[i].is_static = addr_type == MEM_REFERENCE_STATIC;
+	AddressObjectInfo.objects[i].line = line;
+	AddressObjectInfo.objects[i].file_name = filename;
+	AddressObjectInfo.objects[i].name = staticname;
+	AddressObjectInfo.objects[i].module = module;
+	AddressObjectInfo.num_objects++;
+
+#if defined(DEBUG)
+	fprintf (stderr, PACKAGE_NAME": AddressTable_Insert_MemReference (%d, %s, %s, %s, %d) -> %d\n",
+	  addr_type, module, staticname, filename, line, i);
+#endif
+
+	return i;
+}
+#endif /* HAVE_DEFINED(BFD) */
+
 /** AddressTable_Insert
  *
  * @param address
@@ -512,6 +694,11 @@ static int AddressTable_Insert (UINT64 address, int addr_type, char *module,
 	struct function_table * FuncTab;
 	int function_id = 0;
 
+#if defined(DEBUG)
+	fprintf (stderr, PACKAGE_NAME": AddressTable_Insert (%lx, %d, %s, %s, %s, %d)\n",
+	  address, addr_type, module, funcname, filename, line);
+#endif
+
 	AddrTab = AddressTable[addr_type];
 	FuncTab = FunctionTable[addr_type];
 
@@ -570,6 +757,19 @@ static int AddressTable_Insert (UINT64 address, int addr_type, char *module,
 	return new_address_id;
 }
 
+#if defined(HAVE_BFD)
+static void Translate_Address_Data (UINT64 address, unsigned ptask, unsigned task,
+	char **symbol)
+{
+	*symbol = ADDR_UNRESOLVED;
+
+	if (!Translate_Addresses) 
+		return;
+
+	ObjectTable_GetSymbolFromAddress (address, ptask, task, symbol);
+}
+#endif /* HAVE_BFD */
+
 #if defined(HAVE_BFD)
 static void Translate_Address (UINT64 address, unsigned ptask, unsigned task,
 	char **module, char ** funcname, char ** filename, int * line)
@@ -588,6 +788,7 @@ static void Translate_Address (UINT64 address, unsigned ptask, unsigned task,
 		return;
 
 	obj = ObjectTable_GetBinaryObjectAt (ptask, task, address);
+
 # if defined(DEBUG)
 	if (obj)
 	{
@@ -597,6 +798,7 @@ static void Translate_Address (UINT64 address, unsigned ptask, unsigned task,
 	else
 		printf ("obj = NULL for address %llx\n", address);
 # endif
+
 	if (obj)
 	{
 		found = BFDmanager_translateAddress (obj->bfdImage, obj->bfdSymbols,
@@ -802,6 +1004,52 @@ void Address2Info_Write_MPI_Labels (FILE * pcf_fd, int uniqueid)
 	}
 }
 
+void Address2Info_Write_MemReferenceCaller_Labels (FILE * pcf_fd)
+{
+	int i;
+	char short_label[1+SHORT_STRING_PREFIX+SHORT_STRING_SUFFIX+strlen(SHORT_STRING_INFIX)];
+
+	if (Address2Info_Initialized())
+	{
+		fprintf(pcf_fd, "%s\n", TYPE_LABEL);
+		fprintf(pcf_fd, "0    %d    %s\n", SAMPLING_ADDRESS_ALLOCATED_OBJECT_EV,
+		  SAMPLING_ADDRESS_ALLOCATED_OBJECT_LBL);
+
+		if (AddressObjectInfo.num_objects > 0)
+			fprintf (pcf_fd, "%s\n0   %s\n", VALUES_LABEL, EVT_END_LBL);
+
+		for (i = 0; i < AddressObjectInfo.num_objects; i++)
+		{
+			struct address_object_info_st *obj = &(AddressObjectInfo.objects[i]);
+
+			if (obj->is_static)
+			{
+				int shortened = ExtraeUtils_shorten_string (SHORT_STRING_PREFIX,
+				  SHORT_STRING_SUFFIX, SHORT_STRING_INFIX,
+				  sizeof(short_label), short_label, obj->name);
+				if (!shortened)
+					fprintf (pcf_fd, "%d %s\n", i+1, obj->name);
+				else
+					fprintf (pcf_fd, "%d %s [%s]\n", i+1, short_label, obj->name);
+			}
+			else
+			{
+				int shortened = ExtraeUtils_shorten_string (SHORT_STRING_PREFIX,
+				  SHORT_STRING_SUFFIX, SHORT_STRING_INFIX,
+				  sizeof(short_label), short_label, obj->file_name);
+				if (!shortened)
+					fprintf (pcf_fd, "%d (%s)\n", i+1, obj->file_name);
+				else
+					fprintf (pcf_fd, "%d (%s) [%s]\n", i+1, short_label,
+					  obj->file_name);
+			}
+		}
+
+		if (AddressObjectInfo.num_objects > 0)
+			LET_SPACES(pcf_fd);
+	}
+}
+
 void Address2Info_Write_OMP_Labels (FILE * pcf_fd, int eventtype,
 	char *eventtype_description, int eventtype_line,
 	char *eventtype_line_description, int uniqueid)
