@@ -51,10 +51,30 @@ void Share_MISC_Operations (void);
 #define CALLOC_LBL                      "calloc()"
 #define REALLOC_LBL                     "realloc()"
 #define FREE_LBL                        "free()"
+#define POSIX_MEMALIGN_LBL		"posix_memalign()"
+#define MEMKIND_MALLOC_LBL              "memkind_malloc()"
+#define MEMKIND_CALLOC_LBL              "memkind_calloc()"
+#define MEMKIND_REALLOC_LBL             "memkind_realloc()"
+#define MEMKIND_POSIX_MEMALIGN_LBL      "memkind_posix_memalign()"
+#define MEMKIND_FREE_LBL                "memkind_free()"
 #define DYNAMIC_MEM_REQUESTED_SIZE_LBL  "Requested size in dynamic memory call"
 #define DYNAMIC_MEM_POINTER_IN_LBL      "In pointer (free, realloc)"
 #define DYNAMIC_MEM_POINTER_OUT_LBL     "Out pointer (malloc, calloc, realloc)"
 
+#define MEMKIND_PARTITION_LBL				"Memkind partition"
+#define MEMKIND_PARTITION_OTHER_LBL			"Other"
+#define MEMKIND_PARTITION_DEFAULT_LBL			"Default"
+#define MEMKIND_PARTITION_HBW_LBL			"HBW"
+#define MEMKIND_PARTITION_HBW_HUGETLB_LBL		"HBW Huge TLB"
+#define MEMKIND_PARTITION_HBW_PREFERRED_LBL		"HBW Preferred"
+#define MEMKIND_PARTITION_HBW_PREFERRED_HUGETLB_LBL	"HBW Preferred Huge TLB"
+#define MEMKIND_PARTITION_HUGETLB_LBL			"Huge TLB"
+#define MEMKIND_PARTITION_HBW_GBTLB_LBL			"HBW GBTLB"
+#define MEMKIND_PARTITION_HBW_PREFERRED_GBTLB_LBL 	"HBW Preferred GBTLB"
+#define MEMKIND_PARTITION_GBTLB_LBL			"GBTLB"
+#define MEMKIND_PARTITION_HBW_INTERLEAVE_LBL		"HBW Interleave"
+#define MEMKIND_PARTITION_INTERLEAVE_LBL		"Interleave"
+
 #define SAMPLING_ADDRESS_LD_LBL             "Sampled address (load)"
 #define SAMPLING_ADDRESS_ST_LBL             "Sampled address (store)"
 #define SAMPLING_ADDRESS_MEM_LEVEL_LBL      "Memory hierarchy location for sampled address"
@@ -64,12 +84,25 @@ void Share_MISC_Operations (void);
 #define SAMPLING_ADDRESS_REFERENCE_COST_LBL "Memory reference cost in core cycles"
 
 #define IO_LBL                          "I/O calls"
+#define OPEN_LBL                        "open()"
+#define FOPEN_LBL                       "fopen()"
 #define READ_LBL                        "read()"
 #define WRITE_LBL                       "write()"
+#define FREAD_LBL                       "fread()"
+#define FWRITE_LBL                      "fwrite()"
+#define PREAD_LBL                       "pread()"
+#define PWRITE_LBL                      "pwrite()"
+#define READV_LBL                       "readv()"
+#define WRITEV_LBL                      "writev()"
+#define PREADV_LBL                      "preadv()"
+#define PWRITEV_LBL                     "pwritev()"
 #define IO_DESCRIPTOR_LBL               "I/O descriptor"
 #define IO_SIZE_LBL                     "I/O size"
 #define IO_DESCRIPTOR_TYPE_LBL          "I/O descriptor type"
+#define FILE_NAME_LBL                   "Filename"
 
 #define CLOCK_FROM_SYSTEM_LBL           "RAW clock() value from system"
 
+#define CPU_EVENT_INTERVAL_LBL          "CPU-Event sampling interval"
+
 #endif
