@@ -212,7 +212,7 @@ static struct t_event_mpit2prv event_mpit2prv[NUM_MPI_PRV_ELEMENTS] = {
 	{MPI_IALLREDUCE_EV, MPITYPE_COLLECTIVE, MPI_IALLREDUCE_VAL, FALSE}, /* 155 */
 	{MPI_IBARRIER_EV, MPITYPE_COLLECTIVE, MPI_IBARRIER_VAL, FALSE},
 	{MPI_IBCAST_EV, MPITYPE_COLLECTIVE, MPI_IBCAST_VAL, FALSE},
-	{MPI_IALLTOALL_EV, MPITYPE_COLLECTIVE, MPI_ALLTOALL_VAL, FALSE},
+	{MPI_IALLTOALL_EV, MPITYPE_COLLECTIVE, MPI_IALLTOALL_VAL, FALSE},
 	{MPI_IALLTOALLV_EV, MPITYPE_COLLECTIVE, MPI_IALLTOALLV_VAL, FALSE},
 	{MPI_IALLGATHER_EV, MPITYPE_COLLECTIVE, MPI_IALLGATHER_VAL, FALSE}, /* 160 */
 	{MPI_IALLGATHERV_EV, MPITYPE_COLLECTIVE, MPI_IALLGATHERV_VAL, FALSE},
@@ -404,7 +404,7 @@ static struct t_prv_val_label mpi_prv_val_label[NUM_MPI_PRV_ELEMENTS] = {
 	{MPI_IBARRIER_VAL, MPI_IBARRIER_LABEL},
 	{MPI_IBCAST_VAL, MPI_IBCAST_LABEL},
 	{MPI_IALLTOALL_VAL, MPI_IALLTOALL_LABEL},
-	{MPI_ALLTOALLV_VAL, MPI_IALLTOALLV_LABEL},
+	{MPI_IALLTOALLV_VAL, MPI_IALLTOALLV_LABEL},
 	{MPI_IALLGATHER_VAL, MPI_IALLGATHER_LABEL},
 	{MPI_IALLGATHER_VAL, MPI_IALLGATHERV_LABEL},
 	{MPI_IGATHER_VAL, MPI_IGATHER_LABEL},
@@ -609,6 +609,13 @@ void MPITEvent_WriteEnabled_MPI_Operations (FILE * fd)
 		  }
 		  fprintf (fd, "%d   %s\n", 0, "Outside MPI");
 		  fprintf (fd, "\n\n");
+
+		  if (prv_block_groups[ii].type == MPITYPE_RMA) //&& ((event_mpit2prv[130].utilitzada) || (event_mpit2prv[131].utilitzada)))
+		  {
+		  	fprintf (fd, "%s\n", "EVENT_TYPE");
+			fprintf (fd, "%d   %d    %s\n\n\n", prv_block_groups[ii].flag_color,
+				MPITYPE_RMA_SIZE, MPITYPE_RMA_SIZE_LABEL);
+		  }
 		}
 	}
 }
