@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/mpi_prv_semantics.c $
- | @last_commit: $Date: 2015-02-25 16:25:28 +0100 (mié, 25 feb 2015) $
- | @version:     $Revision: 3124 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: mpi_prv_semantics.c 3124 2015-02-25 15:25:28Z harald $";
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -88,7 +81,6 @@ static int Get_State (unsigned int EvType)
 		case MPI_FILE_WRITE_AT_ALL_EV:
 			state = STATE_IO;
 		break;
-		case MPI_WIN_CREATE_EV:
 		case MPI_WIN_FREE_EV:
 		case MPI_WIN_START_EV:
 		case MPI_REQUEST_FREE_EV:
@@ -106,6 +98,9 @@ static int Get_State (unsigned int EvType)
 		case MPI_REQUEST_GET_STATUS_EV:
 		case MPI_INTERCOMM_CREATE_EV:
 		case MPI_INTERCOMM_MERGE_EV:
+		case MPI_WIN_POST_EV:
+		case MPI_WIN_COMPLETE_EV:
+		case MPI_WIN_CREATE_EV:
 			state = STATE_MIXED;
 		break;
 		case MPI_PROBE_EV:
@@ -120,6 +115,7 @@ static int Get_State (unsigned int EvType)
 		case MPI_WAITALL_EV:
 		case MPI_WAITSOME_EV:
 		case MPI_WAITANY_EV:
+		case MPI_WIN_WAIT_EV:
 			state = STATE_TWRECV;
 		break;
 		case MPI_SEND_EV:
@@ -135,6 +131,7 @@ static int Get_State (unsigned int EvType)
 			state = STATE_ISEND;
 		break;
 		case MPI_BARRIER_EV:
+		case MPI_IBARRIER_EV:
 			state = STATE_BARRIER;
 		break;
 		case MPI_REDUCE_EV:
@@ -150,6 +147,19 @@ static int Get_State (unsigned int EvType)
 		case MPI_SCATTERV_EV:
 		case MPI_REDUCESCAT_EV:
 		case MPI_SCAN_EV:
+		case MPI_IREDUCE_EV:
+		case MPI_IALLREDUCE_EV:
+		case MPI_IBCAST_EV:
+		case MPI_IALLTOALL_EV:
+		case MPI_IALLTOALLV_EV:
+		case MPI_IALLGATHER_EV:
+		case MPI_IALLGATHERV_EV:
+		case MPI_IGATHER_EV:
+		case MPI_IGATHERV_EV:
+		case MPI_ISCATTER_EV:
+		case MPI_ISCATTERV_EV:
+		case MPI_IREDUCESCAT_EV:
+		case MPI_ISCAN_EV:
 			state = STATE_BCAST;
 		break;
 		case MPI_WIN_FENCE_EV:
@@ -394,18 +404,25 @@ static unsigned int Get_GlobalOP_isRoot (event_t *current, int task)
 	switch (Get_EvEvent(current))
 	{
 		case MPI_REDUCE_EV:
+		case MPI_IREDUCE_EV:
 			res = Get_EvAux(current) == Get_EvTag(current);
 		break;
 		case MPI_BCAST_EV:
+		case MPI_IBCAST_EV:
 			res = Get_EvTarget(current) == Get_EvTag(current);
 		break;
 		case MPI_GATHER_EV:
+		case MPI_IGATHER_EV:
 		case MPI_GATHERV_EV:
+		case MPI_IGATHERV_EV:
 		case MPI_SCATTER_EV:
+		case MPI_ISCATTER_EV:
 		case MPI_SCATTERV_EV:
+		case MPI_ISCATTERV_EV:
 			res = Get_EvTarget(current) == task-1;
 		break;
 		case MPI_REDUCESCAT_EV:
+		case MPI_IREDUCESCAT_EV:
 			res = Get_EvTarget(current) == 0;
 		break;
 	}
@@ -418,12 +435,15 @@ static unsigned int Get_GlobalOP_SendSize (event_t *current, int is_root)
 	switch (Get_EvEvent(current))
 	{
 		case MPI_BARRIER_EV:
+		case MPI_IBARRIER_EV:
 			res = 0;
 		break;
 		case MPI_REDUCE_EV:
+		case MPI_IREDUCE_EV:
 			res = (is_root)?0:Get_EvSize(current);
 		break;
 		case MPI_BCAST_EV:
+		case MPI_IBCAST_EV:
 			res = (!is_root)?0:Get_EvSize(current);
 		break;
 		default:
@@ -439,19 +459,25 @@ static unsigned int Get_GlobalOP_RecvSize (event_t *current, int is_root)
 	switch (Get_EvEvent(current))
 	{
 		case MPI_BARRIER_EV:
+		case MPI_IBARRIER_EV:
 			res = 0;
 		break;
 		case MPI_REDUCE_EV:
+		case MPI_IREDUCE_EV:
 			res = (!is_root)?0:Get_EvSize(current);
 		break;
 		case MPI_BCAST_EV:
+		case MPI_IBCAST_EV:
 			res = (is_root)?0:Get_EvSize(current);
 		break;
 		case MPI_REDUCESCAT_EV:
+		case MPI_IREDUCESCAT_EV:
 			res = (is_root)?Get_EvSize(current):Get_EvAux(current);
 		break;
 		case MPI_SCAN_EV:
+		case MPI_ISCAN_EV:
 		case MPI_ALLREDUCE_EV:
+		case MPI_IALLREDUCE_EV:
 			res = Get_EvSize(current);
 		break;
 		default:
@@ -499,7 +525,7 @@ static int GlobalOP_event (event_t * current_event,
 		send_size = Get_GlobalOP_SendSize (current_event, is_root);
 		receive_size = Get_GlobalOP_RecvSize (current_event, is_root);
 		trace_enter_global_op (cpu, ptask, task, thread, current_time, comm_id,
-		  send_size, receive_size, is_root?1:0, TRUE);
+		  send_size, receive_size, is_root?1:0);
 
 		Enable_MPI_Soft_Counter (EvType);
 	}
@@ -551,6 +577,31 @@ static int Other_MPI_Event (event_t * current_event,
 	return 0;
 }
 
+/******************************************************************************
+ ***  MPIIO_Event:
+ ******************************************************************************/
+
+static int MPIIO_Event (event_t * current_event,
+        unsigned long long current_time, unsigned int cpu, unsigned int ptask,
+        unsigned int task, unsigned int thread, FileSet_t *fset)
+{
+        unsigned int EvType, EvValue;
+        UNREFERENCED_PARAMETER(fset);
+
+        EvType  = Get_EvEvent (current_event);
+        EvValue = Get_EvValue (current_event);
+
+        Switch_State (Get_State(EvType), (EvValue == EVT_BEGIN), ptask, task, thread);
+
+        trace_paraver_state (cpu, ptask, task, thread, current_time);
+        trace_paraver_event (cpu, ptask, task, thread, current_time, EvType, EvValue);
+        trace_paraver_event (cpu, ptask, task, thread, current_time, MPI_IO_SIZE_EV, Get_EvSize( current_event ));
+
+        Enable_MPI_Soft_Counter (EvType);
+
+	return 0;
+}
+
 /******************************************************************************
  ***  Recv_Event
  ******************************************************************************/
@@ -1060,6 +1111,7 @@ int MPI_RMA_Event (event_t * current_event, unsigned long long current_time,
 	unsigned int cpu, unsigned int ptask, unsigned int task, unsigned int thread,
 	FileSet_t *fset)
 {
+	unsigned int EvSize;
 	UNREFERENCED_PARAMETER(fset);
 
 	Switch_State (Get_State(Get_EvEvent(current_event)),
@@ -1069,6 +1121,13 @@ int MPI_RMA_Event (event_t * current_event, unsigned long long current_time,
 	trace_paraver_event (cpu, ptask, task, thread, current_time,
 		Get_EvEvent(current_event), Get_EvValue (current_event));
 
+	EvSize = Get_EvSize (current_event);
+	if (EvSize != 0)
+	{
+		trace_paraver_event (cpu, ptask, task, thread, current_time,
+			MPI_RMA_SIZE, EvSize);
+	}
+
 	return 0;
 }
 
@@ -1147,14 +1206,14 @@ SingleEv_Handler_t PRV_MPI_Event_Handlers[] = {
 	{ MPI_TEST_COUNTER_EV, MPI_TestSoftwareCounter_Event },
 	{ MPI_FILE_OPEN_EV, Other_MPI_Event },
 	{ MPI_FILE_CLOSE_EV, Other_MPI_Event },
-	{ MPI_FILE_READ_EV, Other_MPI_Event },
-	{ MPI_FILE_READ_ALL_EV, Other_MPI_Event },
-	{ MPI_FILE_WRITE_EV, Other_MPI_Event },
-	{ MPI_FILE_WRITE_ALL_EV, Other_MPI_Event },
-	{ MPI_FILE_READ_AT_EV, Other_MPI_Event },
-	{ MPI_FILE_READ_AT_ALL_EV, Other_MPI_Event },
-	{ MPI_FILE_WRITE_AT_EV, Other_MPI_Event },
-	{ MPI_FILE_WRITE_AT_ALL_EV, Other_MPI_Event },
+	{ MPI_FILE_READ_EV, MPIIO_Event },
+	{ MPI_FILE_READ_ALL_EV, MPIIO_Event },
+	{ MPI_FILE_WRITE_EV, MPIIO_Event },
+	{ MPI_FILE_WRITE_ALL_EV, MPIIO_Event },
+	{ MPI_FILE_READ_AT_EV, MPIIO_Event },
+	{ MPI_FILE_READ_AT_ALL_EV, MPIIO_Event },
+	{ MPI_FILE_WRITE_AT_EV, MPIIO_Event },
+	{ MPI_FILE_WRITE_AT_ALL_EV, MPIIO_Event },
 	{ MPI_PUT_EV, MPI_RMA_Event},
 	{ MPI_GET_EV, MPI_RMA_Event},
 	{ MPI_WIN_CREATE_EV, MPI_RMA_Event},
@@ -1164,5 +1223,19 @@ SingleEv_Handler_t PRV_MPI_Event_Handlers[] = {
 	{ MPI_WIN_POST_EV, MPI_RMA_Event},
 	{ MPI_WIN_COMPLETE_EV, MPI_RMA_Event},
 	{ MPI_WIN_WAIT_EV, MPI_RMA_Event},
+	{ MPI_IREDUCE_EV, GlobalOP_event},
+	{ MPI_IALLREDUCE_EV, GlobalOP_event},
+	{ MPI_IBARRIER_EV, GlobalOP_event},
+	{ MPI_IBCAST_EV, GlobalOP_event},
+	{ MPI_IALLTOALL_EV, GlobalOP_event},
+	{ MPI_IALLTOALLV_EV, GlobalOP_event},
+	{ MPI_IALLGATHER_EV, GlobalOP_event},
+	{ MPI_IALLGATHERV_EV, GlobalOP_event},
+	{ MPI_IGATHER_EV, GlobalOP_event},
+	{ MPI_IGATHERV_EV, GlobalOP_event},
+	{ MPI_ISCATTER_EV, GlobalOP_event},
+	{ MPI_ISCATTERV_EV, GlobalOP_event},
+	{ MPI_IREDUCESCAT_EV, GlobalOP_event},
+	{ MPI_ISCAN_EV, GlobalOP_event},
 	{ NULL_EV, NULL }
 };
