@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/paraver/trace_to_prv.c $
- | @last_commit: $Date: 2015-04-23 17:12:21 +0200 (jue, 23 abr 2015) $
- | @version:     $Revision: 3269 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: trace_to_prv.c 3269 2015-04-23 15:12:21Z harald $";
-
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -99,14 +92,13 @@ static char UNUSED rcsid[] = "$Id: trace_to_prv.c 3269 2015-04-23 15:12:21Z hara
 #include "dimemas_generator.h"
 
 #include "mpi_prv_events.h"
-#include "pacx_prv_events.h"
 #include "pthread_prv_events.h"
 #include "openshmem_prv_events.h"
 #include "omp_prv_events.h"
 #include "misc_prv_events.h"
-#include "trt_prv_events.h"
-#include "cuda_prv_events.h"
 #include "opencl_prv_events.h"
+#include "cuda_prv_events.h"
+#include "java_prv_events.h"
 #include "addr2info.h"
 #include "timesync.h"
 #include "vector.h"
@@ -221,7 +213,7 @@ static void HandleStackedType (unsigned ptask, unsigned task, unsigned thread,
  ***  Paraver_ProcessTraceFiles
  ******************************************************************************/
 
-int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
+int Paraver_ProcessTraceFiles (unsigned long nfiles,
 	struct input_t *files, unsigned int num_appl,
 	struct Pair_NodeCPU *NodeCPUinfo, int numtasks, int taskid)
 {
@@ -363,14 +355,28 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 		fprintf (stdout, "mpi2prv: Searching synchronization points...");
 		fflush (stdout);
 	}
-	Search_Synchronization_Times (taskid, numtasks, fset, &StartingTimes, &SynchronizationTimes);
+	Search_Synchronization_Times (taskid, numtasks, fset, &StartingTimes,
+	  &SynchronizationTimes);
 	if (0 == taskid)
 		fprintf (stdout, " done\n");
 
+#if defined(DEBUG)
+	for (i = 0; i < nfiles; i++)
+	{
+		fprintf (stderr, "[DEBUG] SynchronizationTimes[%d] = %llu "
+		                 " files[%d].SpawnOffset = %llu\n",
+		  i, SynchronizationTimes[i], i, files[i].SpawnOffset);
+	}
+#endif
+
 	TimeSync_Initialize (num_appl, num_appl_tasks);
 	for (i = 0; i < nfiles; i++)
 		if (files[i].thread-1 == 0)
-			TimeSync_SetInitialTime (files[i].ptask-1, files[i].task-1, StartingTimes[i], SynchronizationTimes[i] - files[i].SpawnOffset, files[i].node);
+			TimeSync_SetInitialTime (files[i].ptask-1,
+			  files[i].task-1,
+			  StartingTimes[i],
+			  SynchronizationTimes[i] - files[i].SpawnOffset,
+			  files[i].node);
 
 	if (get_option_merge_SincronitzaTasks_byNode())
 	{
@@ -445,8 +451,8 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 #endif
 
 			if (Type == PTHREAD_TYPE || Type == OPENMP_TYPE || Type == MISC_TYPE ||
-			    Type == MPI_TYPE || Type == PACX_TYPE || Type == TRT_TYPE ||
-			    Type == CUDA_TYPE || Type == OPENCL_TYPE || Type == OPENSHMEM_TYPE)
+			    Type == MPI_TYPE || Type == CUDA_TYPE || Type == OPENCL_TYPE ||
+			    Type == OPENSHMEM_TYPE || Type == JAVA_TYPE)
 			{
 				task_t *task_info = GET_TASK_INFO(ptask, task);
 
@@ -464,16 +470,14 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 						Enable_OMP_Operation (EvType);
 					else if (MPI_TYPE == Type)
 						Enable_MPI_Operation (EvType);
-					else if (PACX_TYPE == Type)
-						Enable_PACX_Operation (EvType);
-					else if (TRT_TYPE == Type)
-						Enable_TRT_Operation (EvType);
 					else if (CUDA_TYPE == Type)
 						Enable_CUDA_Operation (EvType);
 					else if (OPENCL_TYPE == Type)
 						Enable_OpenCL_Operation (EvType);
 					else if (OPENSHMEM_TYPE == Type)
 						Enable_OPENSHMEM_Operation (EvType);
+					else if (JAVA_TYPE == Type)
+						Enable_Java_Operation (EvType);
 				}
 				else	
 				{
@@ -532,11 +536,6 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 				error = GenerateAliesComunicator (current_event, current_time, cpu, ptask, task, thread, fset, &tmp_nevents, PRV_SEMANTICS);
 				Enable_MPI_Operation (EvType);
 			}
-			else if (Type == PACX_COMM_ALIAS_TYPE)
-			{
-				error = GenerateAliesComunicator (current_event, current_time, cpu, ptask, task, thread, fset, &tmp_nevents, PRV_SEMANTICS);
-				Enable_PACX_Operation (EvType);
-			}
 		}
 		else
 		{
@@ -553,9 +552,10 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 			pct = ((double) parsed_events)/((double) num_of_events)*100.0f;
 			if (pct > last_pct + 5.0 && pct <= 100.0f)
 			{
-				fprintf (stdout, "%.1lf%% ", pct);
+				fprintf (stdout, "%d%% ", (int) pct);
 				fflush (stdout);
-				last_pct += 5.0;
+				while (last_pct + 5.0 < pct)
+					last_pct += 5.0;
 			}
 		}
 
@@ -675,12 +675,13 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 	}
 #endif
 
-	Paraver_JoinFiles (num_appl, outName, fset, current_time, NodeCPUinfo, numtasks,
-		taskid, records_per_task, get_option_merge_TreeFanOut());
+	Paraver_JoinFiles (num_appl, get_merge_OutputTraceName(),
+	  fset, current_time, NodeCPUinfo, numtasks,
+	  taskid, records_per_task, get_option_merge_TreeFanOut());
 
-	strcpy (envName, outName);
+	strcpy (envName, get_merge_OutputTraceName());
 #ifdef HAVE_ZLIB
-	if (strlen (outName) >= 7 && strncmp (&(outName[strlen (outName) - 7]), ".prv.gz", 7) == 0)
+	if (strlen (envName) >= 7 && strncmp (&(envName[strlen (envName) - 7]), ".prv.gz", 7) == 0)
 	{
 		tmp = &(envName[strlen (envName) - 7]);
 	}
@@ -705,7 +706,8 @@ int Paraver_ProcessTraceFiles (char *outName, unsigned long nfiles,
 
 	if (0 == taskid)
 	{
-		fprintf (stdout, "mpi2prv: Congratulations! %s has been generated.\n", outName);
+		fprintf (stdout, "mpi2prv: Congratulations! %s has been generated.\n",
+		  get_merge_OutputTraceName());
 		fflush (stdout);
 	}
 
