@@ -21,12 +21,6 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/trace_macros.h $
- | @last_commit: $Date: 2015-04-27 15:34:28 +0200 (lun, 27 abr 2015) $
- | @version:     $Revision: 3281 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
-
 #ifndef __TRACE_MACROS_H__
 #define __TRACE_MACROS_H__
 
@@ -51,23 +45,14 @@
 # define TRACING_BITMAP_VALID_EVTARGET(evttarget)                              \
     (((long)evttarget) != MPI_ANY_SOURCE && ((long)evttarget) != MPI_PROC_NULL && ((long)evttarget) != MPI_UNDEFINED) 
 # define COMM_STATS_WRAPPER(x) \
-    MPI_stats_Wrapper (x);
-#elif defined(PACX_SUPPORT)
-# define TRACING_BITMAP_VALID_EVTYPE(evttype) \
-   ((evttype) == PACX_BSEND_EV || (evttype) == PACX_SSEND_EV || (evttype) == PACX_RSEND_EV || \
-    (evttype) == PACX_SEND_EV  || (evttype) == PACX_IRECVED_EV || (evttype) == PACX_RECV_EV)
-# define TRACING_BITMAP_VALID_EVTARGET(evttarget)                              \
-    (((long)evttarget) != MPI_ANY_SOURCE && ((long)evttarget) != MPI_PROC_NULL) 
-# define COMM_STATS_WRAPPER(x) \
-    PACX_stats_Wrapper (x);
+    Extrae_MPI_stats_Wrapper (x);
 #else
 # define TRACING_BITMAP_VALID_EVTYPE(evttype)     (TRUE)
 # define TRACING_BITMAP_VALID_EVTARGET(evttarget) (TRUE)
 # define COMM_STATS_WRAPPER(x)
-#endif /* MPI_SUPPORT, PACX_SUPPORT */
+#endif /* MPI_SUPPORT */
 
 #include "trace_macros_mpi.h"
-#include "trace_macros_pacx.h"
 #include "trace_macros_omp.h"
 
 #define SAMPLE_EVENT_HWC(evttime,evttype,evtvalue)               \
@@ -99,6 +84,36 @@
 	}                                                              \
 }
 
+#define SAMPLE_EVENT_HWC_PARAM(evttime,evttype,evtvalue,evtparam)               \
+{                                                                \
+	event_t evt;                                                   \
+	int thread_id = THREADID;                                      \
+	if (!Buffer_IsFull (SAMPLING_BUFFER(thread_id)))               \
+	{                                                              \
+		evt.time = (evttime);                                        \
+		evt.event = (evttype);                                       \
+		evt.value = (evtvalue);                                      \
+		evt.param.misc_param.param = (unsigned long long) (evtparam); \
+		/* We don't read counters right now */                       \
+		HARDWARE_COUNTERS_READ(thread_id, evt, TRUE);                \
+		BUFFER_INSERT(thread_id, SAMPLING_BUFFER(thread_id), evt);   \
+	}                                                              \
+}
+
+#define SAMPLE_EVENT_NOHWC_PARAM(evttime,evttype,evtvalue,evtparam)             \
+{                                                                \
+	event_t evt;                                                   \
+	int thread_id = THREADID;                                      \
+	if (!Buffer_IsFull (SAMPLING_BUFFER(thread_id)))               \
+	{                                                              \
+		evt.time = (evttime);                                        \
+		evt.event = (evttype);                                       \
+		evt.value = (evtvalue);                                      \
+		evt.param.misc_param.param = (unsigned long long) (evtparam); \
+		HARDWARE_COUNTERS_READ(thread_id, evt, FALSE);               \
+		BUFFER_INSERT(thread_id, SAMPLING_BUFFER(thread_id), evt);   \
+	}                                                              \
+}
 
 #define TRACE_MISCEVENT(evttime,evttype,evtvalue,evtparam)        \
 {                                                                 \
