@@ -21,12 +21,6 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/trace_macros_mpi.h $
- | @last_commit: $Date: 2014-06-04 19:00:47 +0200 (mié, 04 jun 2014) $
- | @version:     $Revision: 2720 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
-
 #ifndef TRACE_MACROS_MPI_H_INCLUDED
 #define TRACE_MACROS_MPI_H_INCLUDED
 
@@ -38,7 +32,7 @@
 	     ( Caller_Count[CALLER_MPI]>0 )  &&          \
 	     ( evtvalue == EVT_BEGIN ) )                 \
 	{                                                \
-		trace_callers (evttime, offset, CALLER_MPI); \
+		Extrae_trace_callers (evttime, offset, CALLER_MPI); \
 	}                                                \
 }
 
@@ -149,6 +143,9 @@
 
 #if defined(SAMPLING_SUPPORT)
 
+/* If sampling is supported, we don't use PAPI_reset each time that the
+   counters are read, so counters are always accumulating */
+
 # define BURSTS_MODE_TRACE_MPIEVENT(thread_id, evttime, evtvalue, offset)  \
 { \
 	event_t burst_begin, burst_end;           \
@@ -160,6 +157,8 @@
 	burst_end.value = 0;                      \
 	if (evtvalue == EVT_BEGIN)                \
 	{ \
+        /* If we enter into MPI, we check whether the given threshold has occurred \
+        last MPI exit. If so, dump a burst record and its metrics */ \
 		if ((burst_end.time - last_mpi_exit_time) > MINIMUM_BURST_DURATION) \
 		{ \
 			COPY_ACCUMULATED_COUNTERS_HERE(thread_id, burst_begin); \
@@ -180,7 +179,12 @@
 	} \
 }
 
-#else /* SAMPLING_SUPPORT */
+#else
+
+/*
+   SAMPLING_SUPPORT not supported! Be careful, this is not the default case.
+   configure typically enables SAMPLING_SUPPORt by default
+ */
 
 # define BURSTS_MODE_TRACE_MPIEVENT(thread_id, evttime, evtvalue, offset)  \
 { \
@@ -193,6 +197,8 @@
 	burst_end.value = 0;                      \
 	if (evtvalue == EVT_BEGIN)                \
 	{ \
+        /* If we enter into MPI, we check whether the given threshold has occurred \
+        last MPI exit. If so, dump a burst record and its metrics */ \
 		if ((burst_end.time - last_mpi_exit_time) > MINIMUM_BURST_DURATION) \
 		{ \
 			if (ACCUMULATED_COUNTERS_INITIALIZED(thread_id)) \
