@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/tracer/wrappers/OMP/omp-common.c $
- | @last_commit: $Date: 2015-04-20 16:32:24 +0200 (lun, 20 abr 2015) $
- | @version:     $Revision: 3250 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: omp-common.c 3250 2015-04-20 14:32:24Z harald $";
-
 #include "omp_probe.h"
 #include "omp-common.h"
 #include "wrapper.h"
@@ -64,7 +57,7 @@ void Extrae_OpenMP_Join_Wait_Exit (void)
 	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_OpenMP_UF_Entry (void *uf)
+void Extrae_OpenMP_UF_Entry (const void *uf)
 {
 	Backend_Enter_Instrumentation (2);
 	Probe_OpenMP_UF_Entry ((UINT64) uf);
@@ -190,13 +183,13 @@ void Extrae_OpenMP_Named_Lock_Entry (void)
 	Probe_OpenMP_Named_Lock_Entry ();
 }
 
-void Extrae_OpenMP_Named_Lock_Exit (void *name)
+void Extrae_OpenMP_Named_Lock_Exit (const void *name)
 {
 	Probe_OpenMP_Named_Lock_Exit (name);
 	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_OpenMP_Named_Unlock_Entry (void *name)
+void Extrae_OpenMP_Named_Unlock_Entry (const void *name)
 {
 	Backend_Enter_Instrumentation (2);
 	Probe_OpenMP_Named_Unlock_Entry (name);
@@ -261,7 +254,7 @@ void Extrae_OpenMP_TaskID (long long id)
 	Probe_OpenMP_TaskID (id);
 }
 
-void Extrae_OpenMP_Task_Entry (void* uf)
+void Extrae_OpenMP_Task_Entry (const void* uf)
 {
 	Backend_Enter_Instrumentation (2);
 	Probe_OpenMP_Task_Entry ((UINT64) uf);
@@ -273,7 +266,7 @@ void Extrae_OpenMP_Task_Exit (void)
 	Backend_Leave_Instrumentation ();
 }
 
-void Extrae_OpenMP_TaskUF_Entry (void* uf)
+void Extrae_OpenMP_TaskUF_Entry (const void* uf)
 {
 	Backend_Enter_Instrumentation (2);
 	Probe_OpenMP_TaskUF_Entry ((UINT64) uf);
@@ -433,10 +426,9 @@ void Extrae_OMPT_OpenMP_TaskUF_Exit (UINT64 taskid)
 	Backend_Leave_Instrumentation();
 }
 
-void Extrae_OMPT_dependence (uint64_t pred_task_id, uint64_t succ_task_id,
-	int type, void *data)
+void Extrae_OMPT_dependence (uint64_t pred_task_id, uint64_t succ_task_id)
 {
-	Probe_OMPT_dependence (pred_task_id, succ_task_id, type, data);
+	Probe_OMPT_dependence (pred_task_id, succ_task_id);
 }
 
 static volatile unsigned Extrae_OpenMP_numInstantiatedTasks = 0;
