@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/branches/2.5/src/tracer/wrappers/OMP/omp_wrapper.c $
- | @last_commit: $Date: 2014-02-20 16:48:43 +0100 (jue, 20 feb 2014) $
- | @version:     $Revision: 2487 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: omp_wrapper.c 2487 2014-02-20 15:48:43Z harald $";
-
 #ifdef HAVE_ASSERT_H
 # include <assert.h>
 #endif
@@ -54,6 +47,7 @@ static char UNUSED rcsid[] = "$Id: omp_wrapper.c 2487 2014-02-20 15:48:43Z haral
 # undef __USE_GNU
 #endif
 
+#include "misc_interface.h"
 #include "misc_wrapper.h"
 #include "wrapper.h"
 #include "threadid.h"
@@ -61,10 +55,11 @@ static char UNUSED rcsid[] = "$Id: omp_wrapper.c 2487 2014-02-20 15:48:43Z haral
 #include "omp_probe.h"
 #include "omp-common.h"
 #include "ompt-helper.h"
+#include "ompt-target.h"
 
 
 // # define EMPTY_OMPT_CALLBACKS /* For Benchmarking purposes */
-//  #define DEBUG
+// #define DEBUG
 // #define DEBUG_THREAD
 
 //*****************************************************************************
@@ -85,7 +80,13 @@ static omptthid_threadid_t *ompt_thids = NULL;
 static unsigned n_ompt_thids = 0;
 static pthread_mutex_t mutex_thids = PTHREAD_MUTEX_INITIALIZER;
 
-void Extrae_OMPT_register_ompt_thread_id (ompt_thread_id_t ompt_thid, unsigned threadid)
+static int ompt_targets_initialized = 0;
+
+/* Extrae_OMPT_register_ompt_thread_id
+   registers a OMPT thread ompt_thread_id_t and assign it a Paraver thread with
+   an identifier threadid */
+static void Extrae_OMPT_register_ompt_thread_id (ompt_thread_id_t ompt_thid,
+	unsigned threadid)
 {
 	int found_empty = FALSE;
 	unsigned u;
@@ -131,16 +132,15 @@ void Extrae_OMPT_register_ompt_thread_id (ompt_thread_id_t ompt_thid, unsigned t
 	pthread_mutex_unlock (&mutex_thids);
 }
 
-void Extrae_OMPT_unregister_ompt_thread_id (ompt_thread_id_t ompt_thid)
+/* Extrae_OMPT_unregister_ompt_thread_id
+   unregisters the OMPT thread with identifier ompt_thid
+*/
+static void Extrae_OMPT_unregister_ompt_thread_id (ompt_thread_id_t ompt_thid)
 {
 	unsigned u;
 
 	pthread_mutex_lock (&mutex_thids);
 
-	ompt_thids = (omptthid_threadid_t*) realloc (ompt_thids,
-	  (n_ompt_thids+1)*sizeof(omptthid_threadid_t));
-	assert (ompt_thids != NULL);
-
 #if defined(DEBUG)
 	printf ("UNREGISTERING(ompt_thid %lu)\n", ompt_thid);
 #endif
@@ -155,7 +155,10 @@ void Extrae_OMPT_unregister_ompt_thread_id (ompt_thread_id_t ompt_thid)
 	pthread_mutex_unlock (&mutex_thids);
 }
 
-unsigned Extrae_OMPT_threadid (void)
+/* Extrae_OMPT_threadid
+   queries to OMPT for the OMPT thread id and then returns for the Paraver
+   registered thread identifier.  */
+static unsigned Extrae_OMPT_threadid (void)
 {
 	ompt_thread_id_t thd = ompt_get_thread_id_fn();
 	unsigned u;
@@ -195,6 +198,11 @@ unsigned Extrae_OMPT_threadid (void)
 	pthread_mutex_unlock (&mutex_thids);
 #endif
 	fprintf (stderr, "OMPTOOL: Failed to search OpenMP(T) thread %lu\n", thd);
+	fprintf (stderr, "OMPTOOL: Registered threads are (%u): ", n_ompt_thids);
+	for (u = 0; u < n_ompt_thids; u++)
+		fprintf (stderr, "ompt thid %lu valid %d ", ompt_thids[u].thid, ompt_thids[u].in_use);
+	fprintf (stderr, "\n");
+
 	assert (1 != 1);
 	return 0;
 }
@@ -227,7 +235,8 @@ unsigned Extrae_OMPT_threadid (void)
 
 static pthread_mutex_t mutex_init_threads = PTHREAD_MUTEX_INITIALIZER;
 
-void OMPT_event_thread_begin (ompt_thread_type_t type, ompt_thread_id_t thid)
+void Extrae_OMPT_event_thread_begin (ompt_thread_type_t type,
+	ompt_thread_id_t thid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	pthread_mutex_lock (&mutex_init_threads);
@@ -239,10 +248,13 @@ void OMPT_event_thread_begin (ompt_thread_type_t type, ompt_thread_id_t thid)
 
 	if (type == ompt_thread_initial)
 	{
+		/* If I'm the initial thread, I'm thread 0 on the Paraver trace */
 		Extrae_OMPT_register_ompt_thread_id (thid, 0);
 	}
 	else
 	{
+		/* If I'm not the initial thread, then look for how many threads are
+		   there and then give it the next id */
 		Extrae_OMPT_register_ompt_thread_id (thid, threads);
 		Backend_ChangeNumberOfThreads (threads + 1);
 	}
@@ -251,7 +263,8 @@ void OMPT_event_thread_begin (ompt_thread_type_t type, ompt_thread_id_t thid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_thread_end (ompt_thread_type_t type, ompt_thread_id_t thid)
+void Extrae_OMPT_event_thread_end (ompt_thread_type_t type,
+	ompt_thread_id_t thid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	pthread_mutex_lock (&mutex_init_threads);
@@ -271,7 +284,8 @@ void OMPT_event_thread_end (ompt_thread_type_t type, ompt_thread_id_t thid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_loop_begin (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_loop_begin (ompt_parallel_id_t pid, ompt_task_id_t tid,
+	const void *wf)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -283,7 +297,7 @@ void OMPT_event_loop_begin (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_loop_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_loop_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -294,44 +308,57 @@ void OMPT_event_loop_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_parallel_begin (ompt_task_id_t tid, ompt_frame_t *ptf, ompt_parallel_id_t pid, uint32_t req_team_size, void *pf)
+void Extrae_OMPT_event_parallel_begin (ompt_task_id_t tid, ompt_frame_t *ptf,
+	ompt_parallel_id_t pid, uint32_t req_team_size, const void *pf,
+	ompt_invoker_t invoker)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(ptf);
 	UNREFERENCED_PARAMETER(tid);
 	UNREFERENCED_PARAMETER(req_team_size);
+	UNREFERENCED_PARAMETER(invoker);
 
 	PROTOTYPE_MESSAGE(" (%ld, %p, %ld, %u, %p)", tid, ptf, pid, req_team_size, pf);
+
+	/* At this point we need to register the parallel region with identifier (pid)
+	   to the outlined code (pointed by pf). This is necessary because there won't
+	   be any further reference to pf but to pid */
 	Extrae_OMPT_register_ompt_parallel_id_pf (pid, pf);
 	Extrae_OpenMP_ParRegion_Entry ();
 	Extrae_OpenMP_EmitTaskStatistics();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_parallel_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_parallel_end (ompt_parallel_id_t pid, ompt_task_id_t tid,
+	ompt_invoker_t invoker)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(tid);
+	UNREFERENCED_PARAMETER(invoker);
 
 	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
+
+	/* At this point we can unregister the association between pid - pf */
 	Extrae_OMPT_unregister_ompt_parallel_id_pf (pid);
 	Extrae_OpenMP_ParRegion_Exit();
 	Extrae_OpenMP_EmitTaskStatistics();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_barrier_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_barrier_begin (ompt_parallel_id_t pid, ompt_task_id_t tid,
+	const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
 	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OpenMP_Barrier_Entry ();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_barrier_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_barrier_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -342,7 +369,8 @@ void OMPT_event_barrier_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_barrier_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_barrier_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -353,7 +381,8 @@ void OMPT_event_wait_barrier_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_barrier_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_barrier_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -364,18 +393,20 @@ void OMPT_event_wait_barrier_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_master_begin(ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_master_begin(ompt_parallel_id_t pid, ompt_task_id_t tid,
+	const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
 	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OMPT_Master_Entry ();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_master_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_master_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -386,31 +417,32 @@ void OMPT_event_master_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_sections_begin (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_sections_begin (ompt_parallel_id_t pid, ompt_task_id_t tid,
+	const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, codeptr_ra);
 	Extrae_OMPT_Sections_Entry();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_sections_end (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_sections_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OMPT_Sections_Exit();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_single_others_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_single_others_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -421,7 +453,8 @@ void OMPT_event_single_others_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_single_others_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_single_others_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -432,31 +465,33 @@ void OMPT_event_single_others_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_single_in_block_begin (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_single_in_block_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid, const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, codeptr_ra);
 	Extrae_OMPT_Single_Entry ();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_single_in_block_end (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_single_in_block_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OMPT_Single_Exit ();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_task_begin (ompt_task_id_t ptid, ompt_frame_t *ptf, ompt_task_id_t tid, void *ntf)
+void Extrae_OMPT_event_task_begin (ompt_task_id_t ptid, const ompt_frame_t *ptf,
+	ompt_task_id_t tid, const void *ntf)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(ptid);
@@ -464,23 +499,31 @@ void OMPT_event_task_begin (ompt_task_id_t ptid, ompt_frame_t *ptf, ompt_task_id
 	UNREFERENCED_PARAMETER(tid);
 
 	PROTOTYPE_MESSAGE(" (%ld, %p, %ld, %p)", ptid, ptf, tid, ntf);
+
+	/* Similar to Extrae_OMPT_event_parallel_begin, ntf won't be available
+	   whenever additional activity to task tid occurs. We need to store the
+	   relation between tid and its code (ntf). In this case, the task is NOT
+	   implicit. */
 	Extrae_OMPT_register_ompt_task_id_tf (tid, ntf, FALSE);
 	Extrae_OpenMP_Notify_NewInstantiatedTask();
 	//Extrae_OpenMP_TaskUF_Entry (ntf); NOTE: Task does not start running here (HSG, for IBM)!
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_task_end (ompt_task_id_t tid)
+void Extrae_OMPT_event_task_end (ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	PROTOTYPE_MESSAGE(" (%ld)", tid);
 	Extrae_OpenMP_Notify_NewExecutedTask();
-	Extrae_OMPT_OpenMP_TaskUF_Exit (tid);
+	if (Extrae_OMPT_tf_task_id_is_running(tid))
+		// If this task was not marked as running at switch, mark it here
+		Extrae_OMPT_OpenMP_TaskUF_Exit (tid);
 	Extrae_OMPT_unregister_ompt_task_id_tf (tid);
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_taskwait_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_taskwait_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -491,7 +534,8 @@ void OMPT_event_wait_taskwait_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_taskwait_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_taskwait_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -502,11 +546,13 @@ void OMPT_event_wait_taskwait_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_taskwait_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_taskwait_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid, const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
 	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OpenMP_Taskwait_Entry ();
@@ -514,7 +560,7 @@ void OMPT_event_taskwait_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_taskwait_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_taskwait_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -526,7 +572,8 @@ void OMPT_event_taskwait_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_taskgroup_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_taskgroup_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -537,7 +584,7 @@ void OMPT_event_taskgroup_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_taskgroup_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_taskgroup_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -548,7 +595,8 @@ void OMPT_event_taskgroup_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_taskgroup_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_taskgroup_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -559,7 +607,8 @@ void OMPT_event_wait_taskgroup_begin (ompt_parallel_id_t pid, ompt_task_id_t tid
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_taskgroup_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_wait_taskgroup_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -570,31 +619,31 @@ void OMPT_event_wait_taskgroup_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_workshare_begin (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_workshare_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid, const void *codeptr_ra)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
+	UNREFERENCED_PARAMETER(codeptr_ra);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, codeptr_ra);
 	Extrae_OMPT_Workshare_Entry();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_workshare_end (ompt_parallel_id_t pid, ompt_task_id_t tid, void *wf)
+void Extrae_OMPT_event_workshare_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
 	UNREFERENCED_PARAMETER(tid);
-	UNREFERENCED_PARAMETER(wf);
 
-	PROTOTYPE_MESSAGE(" (%ld, %ld, %p)", pid, tid, wf);
+	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
 	Extrae_OMPT_Workshare_Exit();
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_idle_begin (ompt_thread_id_t thid)
+void Extrae_OMPT_event_idle_begin (ompt_thread_id_t thid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(thid);
@@ -604,7 +653,7 @@ void OMPT_event_idle_begin (ompt_thread_id_t thid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_idle_end (ompt_thread_id_t thid)
+void Extrae_OMPT_event_idle_end (ompt_thread_id_t thid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(thid);
@@ -614,7 +663,7 @@ void OMPT_event_idle_end (ompt_thread_id_t thid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_lock (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_lock (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -624,7 +673,7 @@ void OMPT_event_release_lock (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_nest_lock_last (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_nest_lock_last (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -634,7 +683,7 @@ void OMPT_event_release_nest_lock_last (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_critical (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_critical (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -645,7 +694,7 @@ void OMPT_event_release_critical (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_ordered (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_ordered (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -655,7 +704,7 @@ void OMPT_event_release_ordered (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_atomic (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_atomic (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -666,18 +715,24 @@ void OMPT_event_release_atomic (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_implicit_task_begin (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_implicit_task_begin (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(tid);
 
 	PROTOTYPE_MESSAGE(" (%ld, %ld)", pid, tid);
+	/* Similar to Extrae_OMPT_event_task_begin but in this case two things differ:
+	   1) we don't have the code associated to the function, so it is derived from the
+	   code associated to the parallel function. 2) We need to mark this task as
+	   implicit task */
 	Extrae_OMPT_register_ompt_task_id_tf (tid, Extrae_OMPT_get_pf_parallel_id(pid), TRUE);
 	Extrae_OpenMP_UF_Entry (Extrae_OMPT_get_pf_parallel_id(pid));
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_implicit_task_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
+void Extrae_OMPT_event_implicit_task_end (ompt_parallel_id_t pid,
+	ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(pid);
@@ -689,7 +744,7 @@ void OMPT_event_implicit_task_end (ompt_parallel_id_t pid, ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_initial_task_begin (ompt_task_id_t tid)
+void Extrae_OMPT_event_initial_task_begin (ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(tid);
@@ -699,7 +754,7 @@ void OMPT_event_initial_task_begin (ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_initial_task_end (ompt_task_id_t tid)
+void Extrae_OMPT_event_initial_task_end (ompt_task_id_t tid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(tid);
@@ -709,10 +764,10 @@ void OMPT_event_initial_task_end (ompt_task_id_t tid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_task_switch (ompt_task_id_t stid, ompt_task_id_t rtid)
+void Extrae_OMPT_event_task_switch (ompt_task_id_t stid, ompt_task_id_t rtid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
-	void *tf;
+	const void *tf;
 	long long taskcounter;
 	int implicit;
 
@@ -721,39 +776,27 @@ void OMPT_event_task_switch (ompt_task_id_t stid, ompt_task_id_t rtid)
 
 	/* Leave a task function if it's not the implicit task. The implicit
 	   task is automatically instrumented elsewhere */
-# if 0
-	if (stid > 0)
-		if ((tf = Extrae_OMPT_get_tf_task_id(stid, NULL)) &&
-	    	 !Extrae_OMPT_get_tf_task_id_is_implicit(stid))
-		Extrae_OMPT_OpenMP_TaskUF_Exit (stid);
-# else
 	if (stid > 0)
 		if ((tf = Extrae_OMPT_get_tf_task_id(stid, &implicit, NULL)))
 			if (!implicit)
+			{
 				Extrae_OMPT_OpenMP_TaskUF_Exit (stid);
-# endif
+				Extrae_OMPT_tf_task_id_set_running (stid, FALSE);
+			}
 
 	/* Enter a task function if it's not the implicit task. The implicit
 	   task is automatically instrumented elsewhere */
-# if 0
-	if ((tf = Extrae_OMPT_get_tf_task_id (rtid, &taskcounter)) &&
-	     !Extrae_OMPT_get_tf_task_id_is_implicit(rtid))
-	{
-		Extrae_OMPT_OpenMP_TaskUF_Entry (tf, rtid);
-		Extrae_OpenMP_TaskID (taskcounter);
-	}
-# else
 	if ((tf = Extrae_OMPT_get_tf_task_id (rtid, &implicit, &taskcounter)))
 		if (!implicit)
 		{
 			Extrae_OMPT_OpenMP_TaskUF_Entry ((UINT64)tf, rtid);
+			Extrae_OMPT_tf_task_id_set_running (rtid, TRUE);
 			Extrae_OpenMP_TaskID (taskcounter);
 		}
-# endif
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_lock (ompt_wait_id_t wid)
+void Extrae_OMPT_event_wait_lock (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -763,7 +806,7 @@ void OMPT_event_wait_lock (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_lock (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_lock (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -773,7 +816,7 @@ void OMPT_event_acquired_lock (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_nest_lock (ompt_wait_id_t wid)
+void Extrae_OMPT_event_wait_nest_lock (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -783,7 +826,7 @@ void OMPT_event_wait_nest_lock (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_nest_lock_first (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_nest_lock_first (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -793,7 +836,7 @@ void OMPT_event_acquired_nest_lock_first (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_release_nest_lock_prev (ompt_wait_id_t wid)
+void Extrae_OMPT_event_release_nest_lock_prev (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -803,7 +846,7 @@ void OMPT_event_release_nest_lock_prev (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_nest_lock_next (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_nest_lock_next (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -813,7 +856,7 @@ void OMPT_event_acquired_nest_lock_next (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_critical (ompt_wait_id_t wid)
+void Extrae_OMPT_event_wait_critical (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -823,7 +866,7 @@ void OMPT_event_wait_critical (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_critical (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_critical (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -834,7 +877,7 @@ void OMPT_event_acquired_critical (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_ordered (ompt_wait_id_t wid)
+void Extrae_OMPT_event_wait_ordered (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -844,7 +887,7 @@ void OMPT_event_wait_ordered (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_ordered (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_ordered (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -854,7 +897,7 @@ void OMPT_event_acquired_ordered (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_wait_atomic (ompt_wait_id_t wid)
+void Extrae_OMPT_event_wait_atomic (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -864,7 +907,7 @@ void OMPT_event_wait_atomic (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_acquired_atomic (ompt_wait_id_t wid)
+void Extrae_OMPT_event_acquired_atomic (ompt_wait_id_t wid)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(wid);
@@ -875,7 +918,7 @@ void OMPT_event_acquired_atomic (ompt_wait_id_t wid)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-void OMPT_event_control (uint64_t command, uint64_t modifier)
+void Extrae_OMPT_event_control (uint64_t command, uint64_t modifier)
 {
 #ifndef EMPTY_OMPT_CALLBACKS
 	UNREFERENCED_PARAMETER(modifier);
@@ -893,20 +936,38 @@ void OMPT_event_control (uint64_t command, uint64_t modifier)
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
-
-void OMPT_event_dependence( /* for new dependence instrumentation */         
+void Extrae_OMPT_event_task_dependence_pair ( /* for new dependence */
 	ompt_task_id_t pred_task_id, /* ID of predecessor task */
-	ompt_task_id_t succ_task_id, /* ID of successor task */
-	ompt_dependence_type_t type, /* Type of dependence */
-	void *data                   /* Pointer to related data */
+	ompt_task_id_t succ_task_id  /* ID of successor task */
 )
 {
 #ifndef EMPTY_OMPT_CALLBACKS
-	UNREFERENCED_PARAMETER(data);
+	PROTOTYPE_MESSAGE(" (pred_task_id = %lx, succ_task_id = %lx)", pred_task_id,
+		succ_task_id);
 
-	PROTOTYPE_MESSAGE(" (pred_task_id = %lx, succ_task_id = %lx, type = %d, data = %p)", pred_task_id, succ_task_id, type, data);
+	Extrae_OMPT_dependence (pred_task_id, succ_task_id);
+#endif /* EMPTY_OMPT_CALLBACKS */
+}
+
+void Extrae_OMPT_event_task_dependences ( /* for new dependences */
+	ompt_task_id_t task_id,        /* ID of task */
+	ompt_task_dependence_t *deps,  /* vector of task dependences */
+	int ndeps
+)
+{
+#ifndef EMPTY_OMPT_CALLBACKS
+	UNREFERENCED_PARAMETER(task_id);
+	UNREFERENCED_PARAMETER(deps);
+	UNREFERENCED_PARAMETER(ndeps);
+
+	PROTOTYPE_MESSAGE(" (task_id = %d, deps = %p, ndeps = %d)", task_id, deps, ndeps);
+#if defined(DEBUG)
+	int i;
+
+	for (i = 0; i < ndeps; i++)
+		printf ("dependence [%d/%d] : variable @ %p, len = %u, flags = %d\n", i+1, ndeps, deps[i].variable_addr);
+#endif
 
-	Extrae_OMPT_dependence (pred_task_id, succ_task_id, type, data);
 #endif /* EMPTY_OMPT_CALLBACKS */
 }
 
@@ -925,67 +986,68 @@ struct OMPT_callbacks_st
 
 static struct OMPT_callbacks_st ompt_callbacks[] =
 {
-	CALLBACK_ENTRY (ompt_event_loop_begin, OMPT_event_loop_begin),
-	CALLBACK_ENTRY (ompt_event_loop_end, OMPT_event_loop_end),
-	CALLBACK_ENTRY (ompt_event_parallel_begin, OMPT_event_parallel_begin),
-	CALLBACK_ENTRY (ompt_event_parallel_end, OMPT_event_parallel_end),
-	CALLBACK_ENTRY (ompt_event_barrier_begin, OMPT_event_barrier_begin),
-	CALLBACK_ENTRY (ompt_event_barrier_end, OMPT_event_barrier_end),
-	CALLBACK_ENTRY (ompt_event_wait_barrier_begin, OMPT_event_wait_barrier_begin),
-	CALLBACK_ENTRY (ompt_event_wait_barrier_end, OMPT_event_wait_barrier_end),
-	CALLBACK_ENTRY (ompt_event_sections_begin, OMPT_event_sections_begin),
-	CALLBACK_ENTRY (ompt_event_sections_end, OMPT_event_sections_end),
-	CALLBACK_ENTRY (ompt_event_task_begin, OMPT_event_task_begin),
-	CALLBACK_ENTRY (ompt_event_task_end, OMPT_event_task_end),
-	// CALLBACK_ENTRY (ompt_event_wait_taskwait_begin, OMPT_event_wait_taskwait_begin),
-	// CALLBACK_ENTRY (ompt_event_wait_taskwait_end, OMPT_event_wait_taskwait_end),
-	CALLBACK_ENTRY (ompt_event_taskwait_begin, OMPT_event_taskwait_begin),
-	CALLBACK_ENTRY (ompt_event_taskwait_end, OMPT_event_taskwait_end),
-	CALLBACK_ENTRY (ompt_event_wait_taskgroup_begin, OMPT_event_wait_taskgroup_begin),
-	CALLBACK_ENTRY (ompt_event_wait_taskgroup_end, OMPT_event_wait_taskgroup_end),
-	CALLBACK_ENTRY (ompt_event_taskgroup_begin, OMPT_event_taskgroup_begin),
-	CALLBACK_ENTRY (ompt_event_taskgroup_end, OMPT_event_taskgroup_end),
-	CALLBACK_ENTRY (ompt_event_workshare_begin, OMPT_event_workshare_begin),
-	CALLBACK_ENTRY (ompt_event_workshare_end, OMPT_event_workshare_end),
-	CALLBACK_ENTRY (ompt_event_idle_begin, OMPT_event_idle_begin),
-	CALLBACK_ENTRY (ompt_event_idle_end, OMPT_event_idle_end),
-	CALLBACK_ENTRY (ompt_event_implicit_task_begin, OMPT_event_implicit_task_begin),
-	CALLBACK_ENTRY (ompt_event_implicit_task_end, OMPT_event_implicit_task_end),
-	CALLBACK_ENTRY (ompt_event_initial_task_begin, OMPT_event_initial_task_begin),
-	CALLBACK_ENTRY (ompt_event_initial_task_end, OMPT_event_initial_task_end),
-	CALLBACK_ENTRY (ompt_event_task_switch, OMPT_event_task_switch),
-	CALLBACK_ENTRY (ompt_event_wait_lock, OMPT_event_wait_lock),
-	CALLBACK_ENTRY (ompt_event_thread_begin, OMPT_event_thread_begin),
-	CALLBACK_ENTRY (ompt_event_thread_end, OMPT_event_thread_end),
-	CALLBACK_ENTRY (ompt_event_control, OMPT_event_control),
-	CALLBACK_ENTRY (ompt_event_dependence, OMPT_event_dependence),
+	CALLBACK_ENTRY (ompt_event_loop_begin, Extrae_OMPT_event_loop_begin),
+	CALLBACK_ENTRY (ompt_event_loop_end, Extrae_OMPT_event_loop_end),
+	CALLBACK_ENTRY (ompt_event_parallel_begin, Extrae_OMPT_event_parallel_begin),
+	CALLBACK_ENTRY (ompt_event_parallel_end, Extrae_OMPT_event_parallel_end),
+	CALLBACK_ENTRY (ompt_event_barrier_begin, Extrae_OMPT_event_barrier_begin),
+	CALLBACK_ENTRY (ompt_event_barrier_end, Extrae_OMPT_event_barrier_end),
+	CALLBACK_ENTRY (ompt_event_wait_barrier_begin, Extrae_OMPT_event_wait_barrier_begin),
+	CALLBACK_ENTRY (ompt_event_wait_barrier_end, Extrae_OMPT_event_wait_barrier_end),
+	CALLBACK_ENTRY (ompt_event_sections_begin, Extrae_OMPT_event_sections_begin),
+	CALLBACK_ENTRY (ompt_event_sections_end, Extrae_OMPT_event_sections_end),
+	CALLBACK_ENTRY (ompt_event_task_begin, Extrae_OMPT_event_task_begin),
+	CALLBACK_ENTRY (ompt_event_task_end, Extrae_OMPT_event_task_end),
+	// CALLBACK_ENTRY (ompt_event_wait_taskwait_begin, Extrae_OMPT_event_wait_taskwait_begin),
+	// CALLBACK_ENTRY (ompt_event_wait_taskwait_end, Extrae_OMPT_event_wait_taskwait_end),
+	CALLBACK_ENTRY (ompt_event_taskwait_begin, Extrae_OMPT_event_taskwait_begin),
+	CALLBACK_ENTRY (ompt_event_taskwait_end, Extrae_OMPT_event_taskwait_end),
+	CALLBACK_ENTRY (ompt_event_wait_taskgroup_begin, Extrae_OMPT_event_wait_taskgroup_begin),
+	CALLBACK_ENTRY (ompt_event_wait_taskgroup_end, Extrae_OMPT_event_wait_taskgroup_end),
+	CALLBACK_ENTRY (ompt_event_taskgroup_begin, Extrae_OMPT_event_taskgroup_begin),
+	CALLBACK_ENTRY (ompt_event_taskgroup_end, Extrae_OMPT_event_taskgroup_end),
+	CALLBACK_ENTRY (ompt_event_workshare_begin, Extrae_OMPT_event_workshare_begin),
+	CALLBACK_ENTRY (ompt_event_workshare_end, Extrae_OMPT_event_workshare_end),
+	CALLBACK_ENTRY (ompt_event_idle_begin, Extrae_OMPT_event_idle_begin),
+	CALLBACK_ENTRY (ompt_event_idle_end, Extrae_OMPT_event_idle_end),
+	CALLBACK_ENTRY (ompt_event_implicit_task_begin, Extrae_OMPT_event_implicit_task_begin),
+	CALLBACK_ENTRY (ompt_event_implicit_task_end, Extrae_OMPT_event_implicit_task_end),
+	CALLBACK_ENTRY (ompt_event_initial_task_begin, Extrae_OMPT_event_initial_task_begin),
+	CALLBACK_ENTRY (ompt_event_initial_task_end, Extrae_OMPT_event_initial_task_end),
+	CALLBACK_ENTRY (ompt_event_task_switch, Extrae_OMPT_event_task_switch),
+	CALLBACK_ENTRY (ompt_event_wait_lock, Extrae_OMPT_event_wait_lock),
+	CALLBACK_ENTRY (ompt_event_thread_begin, Extrae_OMPT_event_thread_begin),
+	CALLBACK_ENTRY (ompt_event_thread_end, Extrae_OMPT_event_thread_end),
+	CALLBACK_ENTRY (ompt_event_control, Extrae_OMPT_event_control),
+	CALLBACK_ENTRY (ompt_event_task_dependences, Extrae_OMPT_event_task_dependences),
+	CALLBACK_ENTRY (ompt_event_task_dependence_pair, Extrae_OMPT_event_task_dependence_pair),
  	{ "empty,", (ompt_event_t) 0, 0 },
  };
  
 struct OMPT_callbacks_st ompt_callbacks_locks[] =
 {
-	CALLBACK_ENTRY (ompt_event_master_begin, OMPT_event_master_begin),
-	CALLBACK_ENTRY (ompt_event_master_end, OMPT_event_master_end),
-	CALLBACK_ENTRY (ompt_event_single_others_begin, OMPT_event_single_others_begin),
-	CALLBACK_ENTRY (ompt_event_single_others_end, OMPT_event_single_others_end),
-	CALLBACK_ENTRY (ompt_event_single_in_block_begin, OMPT_event_single_in_block_begin),
-	CALLBACK_ENTRY (ompt_event_single_in_block_end, OMPT_event_single_in_block_end),
-	CALLBACK_ENTRY (ompt_event_release_lock, OMPT_event_release_lock),
-	CALLBACK_ENTRY (ompt_event_release_nest_lock_last, OMPT_event_release_nest_lock_last),
-	CALLBACK_ENTRY (ompt_event_release_critical, OMPT_event_release_critical),
-	CALLBACK_ENTRY (ompt_event_release_ordered, OMPT_event_release_ordered),
-	CALLBACK_ENTRY (ompt_event_release_atomic, OMPT_event_release_atomic),
-	CALLBACK_ENTRY (ompt_event_acquired_lock, OMPT_event_acquired_lock),
-	CALLBACK_ENTRY (ompt_event_wait_nest_lock, OMPT_event_wait_nest_lock),
-	CALLBACK_ENTRY (ompt_event_acquired_nest_lock_first, OMPT_event_acquired_nest_lock_first),
-	CALLBACK_ENTRY (ompt_event_release_nest_lock_prev, OMPT_event_release_nest_lock_prev),
-	CALLBACK_ENTRY (ompt_event_acquired_nest_lock_next, OMPT_event_acquired_nest_lock_next),
-	CALLBACK_ENTRY (ompt_event_wait_critical, OMPT_event_wait_critical),
-	CALLBACK_ENTRY (ompt_event_acquired_critical, OMPT_event_acquired_critical),
-	CALLBACK_ENTRY (ompt_event_wait_ordered, OMPT_event_wait_ordered),
-	CALLBACK_ENTRY (ompt_event_acquired_ordered, OMPT_event_acquired_ordered),
-	CALLBACK_ENTRY (ompt_event_wait_atomic, OMPT_event_wait_atomic),
-	CALLBACK_ENTRY (ompt_event_acquired_atomic, OMPT_event_acquired_atomic),
+	CALLBACK_ENTRY (ompt_event_master_begin, Extrae_OMPT_event_master_begin),
+	CALLBACK_ENTRY (ompt_event_master_end, Extrae_OMPT_event_master_end),
+	CALLBACK_ENTRY (ompt_event_single_others_begin, Extrae_OMPT_event_single_others_begin),
+	CALLBACK_ENTRY (ompt_event_single_others_end, Extrae_OMPT_event_single_others_end),
+	CALLBACK_ENTRY (ompt_event_single_in_block_begin, Extrae_OMPT_event_single_in_block_begin),
+	CALLBACK_ENTRY (ompt_event_single_in_block_end, Extrae_OMPT_event_single_in_block_end),
+	CALLBACK_ENTRY (ompt_event_release_lock, Extrae_OMPT_event_release_lock),
+	CALLBACK_ENTRY (ompt_event_release_nest_lock_last, Extrae_OMPT_event_release_nest_lock_last),
+	CALLBACK_ENTRY (ompt_event_release_critical, Extrae_OMPT_event_release_critical),
+	CALLBACK_ENTRY (ompt_event_release_ordered, Extrae_OMPT_event_release_ordered),
+	CALLBACK_ENTRY (ompt_event_release_atomic, Extrae_OMPT_event_release_atomic),
+	CALLBACK_ENTRY (ompt_event_acquired_lock, Extrae_OMPT_event_acquired_lock),
+	CALLBACK_ENTRY (ompt_event_wait_nest_lock, Extrae_OMPT_event_wait_nest_lock),
+	CALLBACK_ENTRY (ompt_event_acquired_nest_lock_first, Extrae_OMPT_event_acquired_nest_lock_first),
+	CALLBACK_ENTRY (ompt_event_release_nest_lock_prev, Extrae_OMPT_event_release_nest_lock_prev),
+	CALLBACK_ENTRY (ompt_event_acquired_nest_lock_next, Extrae_OMPT_event_acquired_nest_lock_next),
+	CALLBACK_ENTRY (ompt_event_wait_critical, Extrae_OMPT_event_wait_critical),
+	CALLBACK_ENTRY (ompt_event_acquired_critical, Extrae_OMPT_event_acquired_critical),
+	CALLBACK_ENTRY (ompt_event_wait_ordered, Extrae_OMPT_event_wait_ordered),
+	CALLBACK_ENTRY (ompt_event_acquired_ordered, Extrae_OMPT_event_acquired_ordered),
+	CALLBACK_ENTRY (ompt_event_wait_atomic, Extrae_OMPT_event_wait_atomic),
+	CALLBACK_ENTRY (ompt_event_acquired_atomic, Extrae_OMPT_event_acquired_atomic),
 	{ "empty,", (ompt_event_t) 0, 0 },
 };
  
@@ -996,7 +1058,7 @@ typedef enum {
 	OMPT_UNKNOWN
 } ompt_runtime_t;
 
-int ompt_initialize(
+void ompt_initialize(
 	ompt_function_lookup_t lookup,
 	const char *runtime_version_string, 
 	unsigned ompt_version)
@@ -1007,6 +1069,8 @@ int ompt_initialize(
 
 	UNREFERENCED_PARAMETER(ompt_version);
 
+	Extrae_init();
+
 #if defined(DEBUG) 
 	printf("OMPT IS INITIALIZING: lookup functions with runtime version %s and ompt version %d\n",
 	  runtime_version_string, ompt_version);
@@ -1021,11 +1085,13 @@ int ompt_initialize(
 
 #if defined(DEBUG)
 	printf ("OMPTOOL: ompt_rte = %d\n", ompt_rte);
-#endif
+#endif 
 
+	/* Ask OMPT for the routine ompt_set_callback */
 	ompt_set_callback_fn = (int(*)(ompt_event_t, ompt_callback_t)) lookup("ompt_set_callback");
 	assert (ompt_set_callback_fn != NULL);
 
+	/* Ask OMPT for the routine ompt_get_thread_id */
 	ompt_get_thread_id_fn = (ompt_thread_id_t(*)(void)) lookup("ompt_get_thread_id");
 	assert (ompt_get_thread_id_fn != NULL);
 
@@ -1038,6 +1104,7 @@ int ompt_initialize(
 	i = 0;
 	while (ompt_callbacks[i].evt != (ompt_event_t) 0)
 	{
+		/* What happens to IBM runtime? */
 		if (ompt_rte == OMPT_RTE_IBM)
 		{
 			if (ompt_callbacks[i].evt != ompt_event_master_begin
@@ -1089,9 +1156,32 @@ int ompt_initialize(
 
 	Extrae_set_threadid_function (Extrae_OMPT_threadid);
 
+
+	/* This point starts support for OMPT accelerators. */
+	ompt_targets_initialized = ompt_target_initialize(lookup);
+		
 	UNREFERENCED_PARAMETER(r);
+}
 
+/**
+ * ompt_finalize
+ *
+ * Called from Backend_Finalize when there's support for OpenMP and OMPT enabled to 
+ * make the final flush of the devices' buffers. The finalization has to be started 
+ * by Extrae, because when the OmpSs/OMPT runtime finalizes, Extrae has already closed, 
+ * and it is too late to flush the buffers.
+ */
+void ompt_finalize()
+{
+  /* Check whether the tracing for devices has been enabled */
+  if (ompt_targets_initialized) 
+  {
+    ompt_target_finalize();
+  }
+} 
 
-	return 1;
+ompt_initialize_fn_t ompt_tool (void)
+{
+	return ompt_initialize;
 }
 
