@@ -78,6 +78,20 @@ show_opts() {
                                             Default: ${DEFAULT_QUEUE}
     --reservation=<name>                    Reservation to use when submitting the job. 
                                             Default: ${DEFAULT_RESERVATION}
+EOT
+   if [ -z "${DISABLE_QARG_CONSTRAINTS}" ] || [ "${DISABLE_QARG_CONSTRAINTS}" == "false" ]; then
+    cat <<EOT
+    --constraints=<constraints>		    Constraints to pass to queue system.
+					    Default: ${DEFAULT_CONSTRAINTS}  
+EOT
+  fi
+  if [ -z "${DISABLE_QARG_QOS}" ] || [ "${DISABLE_QARG_QOS}" == "false" ]; then
+    cat <<EOT
+    --qos=<qos>				    Quality of Service to pass to the queue system.
+					    Default: ${DEFAULT_QOS}
+EOT
+  fi
+    cat <<EOT 
     --job_dependency=<jobID>                Postpone job execution until the job dependency has ended.
                                             Default: ${DEFAULT_DEPENDENCY_JOB}
     --storage_home=<string>                 Root installation dir of the storage implementation
@@ -140,6 +154,12 @@ log_args() {
   echo "GPUs per node:             ${gpus_per_node}"
   echo "Job dependency:            ${dependencyJob}"
   echo "Exec-Time:                 ${wc_limit}"
+  if [ -z "${DISABLE_QARG_QOS}" ] || [ "${DISABLE_QARG_QOS}" == "false" ]; then
+    echo "QoS:                       ${qos}"
+  fi
+  if [ -z "${DISABLE_QARG_CONSTRAINTS}" ] || [ "${DISABLE_QARG_CONSTRAINTS}" == "false" ]; then
+    echo "Constraints:               ${constraints}"
+  fi
   echo "Storage Home:              ${storage_home}"
   echo "Storage Properties:        ${storage_props}"
   local other=$(echo ${args_pass} | sed 's/\ --/\n\t\t\t--/g')
@@ -246,6 +266,12 @@ get_args() {
           reservation=*)
             reservation=$(echo $OPTARG | sed -e 's/reservation=//g')
             ;;
+	  qos=*)
+            qos=$(echo $OPTARG | sed -e 's/qos=//g')
+            ;;
+	  constraints=*)
+            constraints=$(echo $OPTARG | sed -e 's/constraints=//g')
+            ;;
           job_dependency=*)
             dependencyJob=$(echo $OPTARG | sed -e 's/job_dependency=//g')
             ;;
@@ -315,6 +341,14 @@ check_args() {
     reservation=${DEFAULT_RESERVATION}
   fi
 
+  if [ -z "${constraints}" ]; then
+    constraints=${DEFAULT_CONSTRAINTS}
+  fi
+  
+  if [ -z "${qos}" ]; then
+    qos=${DEFAULT_QOS}
+  fi
+
   if [ -z "${dependencyJob}" ]; then
     dependencyJob=${DEFAULT_DEPENDENCY_JOB}
   fi
@@ -468,12 +502,36 @@ EOT
     fi
   fi
 
+  # QoS
+  if [ -n "${QARG_QOS}" ]; then
+    if [ "${qos}" != "default" ]; then
+      if [ -z "${DISABLE_QARG_QOS}" ] || [ "${DISABLE_QARG_QOS}" == "false" ]; then
+      	cat >> $TMP_SUBMIT_SCRIPT << EOT
+#${QUEUE_CMD} ${QARG_QOS}${QUEUE_SEPARATOR}${qos}
+EOT
+      fi
+    fi
+  fi
+  
+  # Constraints
+  if [ -n "${QARG_CONSTRAINTS}" ]; then
+    if [ "${constraints}" != "disabled" ]; then
+      if [ -z "${DISABLE_QARG_CONSTRAINTS}" ] || [ "${DISABLE_QARG_CONSTRAINTS}" == "false" ]; then
+        cat >> $TMP_SUBMIT_SCRIPT << EOT
+#${QUEUE_CMD} ${QARG_CONSTRAINTS}${QUEUE_SEPARATOR}${constraints}
+EOT
+      fi
+    fi
+  fi
+
   # Node memory
   if [ -n "${QARG_MEMORY}" ]; then
     if [ "${node_memory}" != "disabled" ]; then
-      cat >> $TMP_SUBMIT_SCRIPT << EOT
+      if [ -z "${DISABLE_QARG_MEMORY}" ] || [ "${DISABLE_QARG_MEMORY}" == "false" ]; then
+          cat >> $TMP_SUBMIT_SCRIPT << EOT
 #${QUEUE_CMD} ${QARG_MEMORY}${QUEUE_SEPARATOR}${node_memory}
 EOT
+      fi
     fi
   fi
 
