@@ -13,8 +13,8 @@
 
   # BLAUNCH start ----------------------------------------------------
   # Check that the current machine has not already awaken any WORKER in PORT and for app UUID
-  worker_class="integratedtoolkit.nio.worker.NIOWorker"
-  pid=$(ps -elfa | grep ${worker_class} | grep ${appUuid} | grep ${worker_port} | grep -v grep | awk '{print $4}')
+  worker_class="es.bsc.compss.nio.worker.NIOWorker"
+  pid=$(ps -elfa | grep ${worker_class} | grep ${appUuid} | grep ${hostName} | grep ${worker_port} | grep -v grep | awk '{print $4}')
   if [ "$pid" != "" ]; then
     if [ "$debug" == "true" ]; then
        echo "Worker already awaken. Nothing to do"
@@ -37,7 +37,7 @@
       echo "Cmd: $cmd ${paramsToCOMPSsWorker}"
   fi
 
-  nohup $cmd ${paramsToCOMPSsWorker} 1>$workingDir/log/worker_${hostName}.out 2> $workingDir/log/worker_${hostName}.err | echo $! &
+  setsid $cmd ${paramsToCOMPSsWorker} 1>$workingDir/log/worker_${hostName}.out 2> $workingDir/log/worker_${hostName}.err < /dev/null | echo $! &
   endCode=$?
 
   post_launch
