@@ -101,7 +101,7 @@
     if [ ! -d $workingDir ]; then
   	/bin/mkdir -p $workingDir
     fi
-    export IT_WORKING_DIR=$workingDir
+    export COMPSS_WORKING_DIR=$workingDir
     mkdir -p $workingDir/log
     mkdir -p $workingDir/jobs
 
@@ -117,11 +117,11 @@
 
     # Set lib path
     if [ "$libPath" != "null" ]; then
-  	    export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$libPath
+        export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$libPath
     fi
   
     # Set appDir
-    export IT_APP_DIR=$appDir
+    export COMPSS_APP_DIR=$appDir
     if [ "$appDir" != "null" ]; then
     	add_to_classpath "$appDir"
     	add_to_classpath "$appDir/lib"
@@ -131,13 +131,18 @@
     if [ "$cp" == "null" ]; then
   	cp=""
     fi
+
+    # Export environment
+    export CLASSPATH=$cpNW:$CLASSPATH
+    export PYTHONPATH=$pythonpath:$PYTHONPATH
+    export LD_LIBRARY_PATH=$libPathNW:$LD_LIBRARY_PATH
   }
   
   setup_jvm() {
     # Prepare the worker command
     local JAVA=java
     local worker_jar=${scriptDir}/../../../../adaptors/nio/worker/compss-adaptors-nio-worker.jar
-    local main_worker_class=integratedtoolkit.nio.worker.NIOWorker
+    local main_worker_class=es.bsc.compss.nio.worker.NIOWorker
     cmd=$JAVA" \
       ${jvmFlags} \
       -XX:+PerfDisableSharedMem \
@@ -146,7 +151,7 @@
       -XX:+UseThreadPriorities \
       -XX:ThreadPriorityPolicy=42 \
       -Dlog4j.configurationFile=${installDir}/Runtime/configuration/log/${itlog4j_file} \
-      -classpath $cp:$CLASSPATH:${worker_jar} \
+      -classpath $CLASSPATH:${worker_jar} \
       ${main_worker_class}"
   }
   
